"""
供 torch 使用的主观逻辑计算; batch; 向量化计算

### bijective mapping

evidence <-> opinion[belief, uncertainty, base_rates]
evicence <-> dirichlet[alpha]
opinion <-> dirichlet

### opinion fusion

- CBF
- ABF
- WBF
- DST (Dempster-Shafer Theory)

### trust

- trust_discounting

### regulation

- degree of conflict

"""

import typing

from typing import TYPE_CHECKING, Literal, Optional, Tuple, Union

import torch

if TYPE_CHECKING:
    from torch import Tensor


OpinionTuple = Tuple['Tensor', 'Tensor', 'Tensor']


def numerical_validate(
    belief_masses: 'Tensor',
    uncertainty: 'Tensor',
    base_rates: 'Tensor',
    auto_normalize: bool = True,
    eps: Optional[float] = None,
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """
    确保主观逻辑意见参数的数值正确性

    参考 subjective_logic.py 中的 __validate 方法实现，针对 torch 张量进行批量验证和归一化处理。

    Parameters
    ----------
    - `belief_masses` : `Tensor`
        - 信念质量张量，形状为 `[batch, n_states]`
    - `uncertainty` : `Tensor`
        - 不确定性张量，形状为 `[batch]`
    - `base_rates` : `Tensor`
        - 基率张量，形状为 `[batch, n_states]` 或 `[n_states]`
    - `auto_normalize` : `bool`, optional
        - 是否自动归一化处理微小的浮点误差，默认为 True
    - `eps` : `float`, optional
        - 数值精度阈值。如果为 None，则根据数据类型自动设置
        - torch.float32: 1e-7, torch.float64: 1e-12

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - 验证并可能归一化后的 (belief_masses, uncertainty, base_rates)

    Raises
    ------
    ValueError
        当参数不满足约束条件且无法通过归一化修正时
    """
    dtype = belief_masses.dtype
    batch_size, n_states = belief_masses.shape

    # 设置数值精度阈值
    if eps is None:
        eps = 1e-7 if dtype == torch.float32 else 1e-12

    # 1. 基本约束检查
    if torch.any(uncertainty < 0) or torch.any(uncertainty > 1.0):
        raise ValueError('不确定性必须在 [0,1] 范围内')

    if torch.any(belief_masses < 0) or torch.any(belief_masses > 1.0):
        raise ValueError('所有信念质量必须在 [0,1] 范围内')

    if n_states < 2:
        raise ValueError('意见至少需要2个状态')

    # 2. 广播 base_rates 到正确的形状
    if base_rates.dim() == 1:
        base_rates = base_rates.unsqueeze(0).expand(batch_size, -1)

    if base_rates.shape != belief_masses.shape:
        raise ValueError('信念质量和基率维度必须相同')

    # 3. 验证并归一化 belief_masses 和 uncertainty
    total = torch.sum(belief_masses, dim=1) + uncertainty  # [batch]
    total_deviation = torch.abs(total - 1.0)  # [batch]

    # 检查是否有偏差过大的样本
    invalid_mask = total_deviation > eps
    if torch.any(invalid_mask):
        max_deviation = torch.max(total_deviation)
        raise ValueError(
            f'信念质量与不确定性之和不等于1，最大偏差：{max_deviation.item():.2e}'
        )

    # 自动归一化微小的浮点误差
    if auto_normalize:
        normalize_mask = total_deviation > 0
        if torch.any(normalize_mask):
            # 只对需要归一化的样本进行处理
            total_expanded = total.unsqueeze(1)  # [batch, 1]
            belief_masses = torch.where(
                normalize_mask.unsqueeze(1), belief_masses / total_expanded, belief_masses
            )
            uncertainty = torch.where(normalize_mask, uncertainty / total, uncertainty)

    # 4. 验证并归一化 base_rates
    base_rates_sum = torch.sum(base_rates, dim=1)  # [batch]
    base_rates_deviation = torch.abs(base_rates_sum - 1.0)  # [batch]

    # 检查是否有偏差过大的样本
    invalid_base_rates_mask = base_rates_deviation > eps
    if torch.any(invalid_base_rates_mask):
        max_base_rates_deviation = torch.max(base_rates_deviation)
        raise ValueError(
            f'基率之和不等于1，最大偏差：{max_base_rates_deviation.item():.2e}'
        )

    # 自动归一化基率的微小浮点误差
    if auto_normalize:
        normalize_base_rates_mask = base_rates_deviation > 0
        if torch.any(normalize_base_rates_mask):
            base_rates_sum_expanded = base_rates_sum.unsqueeze(1)  # [batch, 1]
            base_rates = torch.where(
                normalize_base_rates_mask.unsqueeze(1),
                base_rates / base_rates_sum_expanded,
                base_rates,
            )

    # 5. 确保数值非负
    belief_masses = torch.clamp(belief_masses, min=0.0)
    uncertainty = torch.clamp(uncertainty, min=0.0)
    base_rates = torch.clamp(base_rates, min=0.0)

    return belief_masses, uncertainty, base_rates


def evidence_to_dirichlet(
    evidence: 'Tensor', base_rates: Optional['Tensor'] = None, W: Optional[float] = None
) -> 'Tensor':
    """Evidence 向量转换为狄利克雷参数 alpha

    转换公式: α_i = e_i + W * a_i

    Parameters
    ----------
    - `evidence` : `Tensor`
        - 证据张量，形状为 `[batch, n_states]`
    - `base_rates` : `Tensor`, optional
        - 基率张量，形状为 `[n_states]` 或 `[batch, n_states]`，默认为均匀分布
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tensor`
        - Dirichlet 参数 alpha，形状为 `[batch, n_states]`
    """
    batch_size, n_states = evidence.shape
    device, dtype = evidence.device, evidence.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    if base_rates is None:
        base_rates = torch.ones(n_states, device=device, dtype=dtype) / n_states

    # 广播处理
    if base_rates.dim() == 1:
        base_rates = base_rates.unsqueeze(0).expand(batch_size, -1)

    return evidence + W * base_rates


def dirichlet_to_evidence(
    alpha: 'Tensor', base_rates: Optional['Tensor'] = None, W: Optional[float] = None
) -> 'Tensor':
    """狄利克雷参数转换为 Evidence 向量

    转换公式: e_i = α_i - W * a_i

    Parameters
    ----------
    - `alpha` : `Tensor`
        - Dirichlet 参数，形状为 `[batch, n_states]`
    - `base_rates` : `Tensor`, optional
        - 基率张量，形状为 `[n_states]` 或 `[batch, n_states]`，默认为均匀分布
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tensor`
        - 证据张量，形状为 `[batch, n_states]`
    """
    batch_size, n_states = alpha.shape
    device, dtype = alpha.device, alpha.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    if base_rates is None:
        base_rates = torch.ones(n_states, device=device, dtype=dtype) / n_states

    # 广播处理
    if base_rates.dim() == 1:
        base_rates = base_rates.unsqueeze(0).expand(batch_size, -1)

    evidence = alpha - W * base_rates
    return torch.clamp(evidence, min=0.0)  # 确保非负


def evidence_to_opinion(
    evidence: 'Tensor', base_rates: Optional['Tensor'] = None, W: Optional[float] = None
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """Evidence 向量转换为 SL 意见

    转换公式:
    - belief_masses = evidence / (sum(evidence) + W)
    - uncertainty = W / (sum(evidence) + W)

    Parameters
    ----------
    - `evidence` : `Tensor`
        - 证据张量，形状为 `[batch, n_states]`
    - `base_rates` : `Tensor`, optional
        - 基率张量，形状为 `[n_states]` 或 `[batch, n_states]`，默认为均匀分布
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - (belief_masses, uncertainty, base_rates) - 形状分别为 `[batch, n_states]`, `[batch]`, `[batch, n_states]`
    """
    batch_size, n_states = evidence.shape
    device, dtype = evidence.device, evidence.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    if base_rates is None:
        base_rates = torch.ones(n_states, device=device, dtype=dtype) / n_states

    # 广播处理
    if base_rates.dim() == 1:
        base_rates = base_rates.unsqueeze(0).expand(batch_size, -1)

    total_evidence = torch.sum(evidence, dim=1)  # [batch]
    kappa = total_evidence + W  # [batch]

    belief_masses = evidence / kappa.unsqueeze(1)  # [batch, n_states]
    uncertainty = W / kappa  # u = W / (Σe + W)

    return belief_masses, uncertainty, base_rates


def opinion_to_evidence(
    belief_masses: 'Tensor', uncertainty: 'Tensor', W: Optional[float] = None
) -> 'Tensor':
    """SL 意见转换为 Evidence 向量

    转换公式: e_i = W * b_i / u

    Parameters
    ----------
    - `belief_masses` : `Tensor`
        - 信念质量张量，形状为 `[batch, n_states]`
    - `uncertainty` : `Tensor`
        - 不确定性张量，形状为 `[batch]`
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tensor`
        - 证据张量，形状为 `[batch, n_states]`
    """
    batch_size, n_states = belief_masses.shape

    # 默认参数处理
    if W is None:
        W = float(n_states)

    # 数值稳定性处理
    eps = 1e-12 if belief_masses.dtype == torch.float64 else 1e-7
    denominator = torch.clamp(uncertainty, min=eps)  # [batch]

    return (W * belief_masses) / denominator.unsqueeze(1)


def dirichlet_to_opinion(
    alpha: 'Tensor', base_rates: Optional['Tensor'] = None, W: Optional[float] = None
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """狄利克雷参数转换为 SL 意见

    Parameters
    ----------
    - `alpha` : `Tensor`
        - Dirichlet 参数，形状为 `[batch, n_states]`
    - `base_rates` : `Tensor`, optional
        - 基率张量，形状为 `[n_states]` 或 `[batch, n_states]`，默认为均匀分布
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - (belief_masses, uncertainty, base_rates) - 形状分别为 `[batch, n_states]`, `[batch]`, `[batch, n_states]`
    """
    # alpha -> evidence -> opinion
    evidence = dirichlet_to_evidence(alpha, base_rates, W)
    return evidence_to_opinion(evidence, base_rates, W)


def opinion_to_dirichlet(
    belief_masses: 'Tensor',
    uncertainty: 'Tensor',
    base_rates: 'Tensor',
    W: Optional[float] = None,
) -> 'Tensor':
    """SL 意见转换为狄利克雷参数

    Parameters
    ----------
    - `belief_masses` : `Tensor`
        - 信念质量张量，形状为 `[batch, n_states]`
    - `uncertainty` : `Tensor`
        - 不确定性张量，形状为 `[batch]`
    - `base_rates` : `Tensor`
        - 基率张量，形状为 `[batch, n_states]`
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tensor`
        - Dirichlet 参数 alpha，形状为 `[batch, n_states]`
    """
    # opinion -> evidence -> alpha
    evidence = opinion_to_evidence(belief_masses, uncertainty, W)
    return evidence_to_dirichlet(evidence, base_rates, W)


def evidence_strength(
    input_data: Union['Tensor', OpinionTuple],
    input_type: Literal['evidence', 'opinion', 'alpha'],
    W: Optional[float] = None,
) -> 'Tensor':
    """计算证据强度 (Evidence Strength)，也称为狄利克雷强度 (Dirichlet Strength) S 或 κ.

    证据强度是支撑一个主观意见的所有证据的总和，反映了该意见的确定性程度。
    强度越高，意见越确定（即不确定性越低）。
    其计算公式为: S = Σα = Σe + W。

    Parameters
    ----------
    - `input_data` : `Union[Tensor, Tuple[Tensor, Tensor, Tensor]]`
        - 输入的数据，其具体类型由 `input_type` 参数指定。
    - `input_type` : `Literal['evidence', 'opinion', 'alpha']`
        - 指定 `input_data` 的类型。
        - 'alpha': 输入为 Dirichlet 参数张量，形状 `[batch, n_states]`。
        - 'evidence': 输入为证据张量，形状 `[batch, n_states]`。
        - 'opinion': 输入为 SL 意见元组 `(belief, uncertainty, base_rates)`。
    - `W` : `float`, optional
        - 非信息先验权重。如果 `input_type` 是 'evidence' 或 'opinion'，则会用到此参数。
        - 如果不提供，将根据输入的维度自动推断为 n_states。

    Returns
    -------
    - `Tensor`
        - 证据强度，形状为 `[batch]`

    Raises
    ------
    - `ValueError`
        - 当 `input_type` 不是支持的类型时。
    """
    if input_type == 'alpha':
        # 从 alpha 计算: S = Σα
        alpha = typing.cast('Tensor', input_data)
        return torch.sum(alpha, dim=-1)

    elif input_type == 'evidence':
        # 从 evidence 计算: S = Σe + W
        evidence = typing.cast('Tensor', input_data)
        _, n_states = evidence.shape
        if W is None:
            W = float(n_states)
        total_evidence = torch.sum(evidence, dim=1)
        return total_evidence + W

    elif input_type == 'opinion':
        # 从 opinion 计算: S = W / u
        belief_masses, uncertainty, _ = typing.cast(OpinionTuple, input_data)
        _, n_states = belief_masses.shape
        if W is None:
            W = float(n_states)

        # 数值稳定性处理，防止不确定性为0
        eps = 1e-12 if uncertainty.dtype == torch.float64 else 1e-7
        denominator = torch.clamp(uncertainty, min=eps)
        return W / denominator

    else:
        raise ValueError(
            f"不支持的 input_type: '{input_type}'。 "
            "有效选项为 'evidence', 'opinion', 'alpha'。"
        )


def projected_probabilities(
    belief_masses: 'Tensor', uncertainty: 'Tensor', base_rates: 'Tensor'
) -> 'Tensor':
    """计算投影概率分布

    转换公式: P(x_i) = b_i + a_i * u

    Parameters
    ----------
    - `belief_masses` : `Tensor`
        - 信念质量张量，形状为 `[batch, n_states]`
    - `uncertainty` : `Tensor`
        - 不确定性张量，形状为 `[batch]`
    - `base_rates` : `Tensor`
        - 基率张量，形状为 `[batch, n_states]`

    Returns
    -------
    - `Tensor`
        - 投影概率分布，形状为 `[batch, n_states]`
    """
    return belief_masses + base_rates * uncertainty.unsqueeze(1)


def projected_probability(
    belief_masses: 'Tensor',
    uncertainty: 'Tensor',
    base_rates: 'Tensor',
    belief_idx: int = 0,
) -> 'Tensor':
    """计算二元意见的投影概率（默认以 idx=0 为 belif 状态, idx=1 为 disbelief 状态）

    仅适用于二元意见 (n_states=2)

    Parameters
    ----------
    - `belief_masses` : `Tensor`
        - 信念质量张量，形状为 `[batch, 2]`
    - `uncertainty` : `Tensor`
        - 不确定性张量，形状为 `[batch]`
    - `base_rates` : `Tensor`
        - 基率张量，形状为 `[batch, 2]`
    - `belief_idx` : `int`
        - 意见状态的索引，默认为 0 (以第一个状态为 belief 状态)

    Returns
    -------
    - `Tensor`
        - 第一个状态的投影概率，形状为 `[batch]`
    """
    if belief_masses.shape[1] != 2:
        raise ValueError('projected_probability 仅适用于二元意见')

    probs = projected_probabilities(belief_masses, uncertainty, base_rates)
    return probs[:, belief_idx]


# ========================= Opinion Fusion ========================= #


def cumulative_fusion(
    sources: typing.Sequence[Tuple['Tensor', 'Tensor', 'Tensor']],
    W: Optional[float] = None,
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """累积融合 (Cumulative Belief Fusion, CBF)：组合多个独立信息源的证据

    Parameters
    ----------
    - `sources` : `list[Tuple[Tensor, Tensor, Tensor]]`
        - 要融合的意见来源，每个意见为 (belief_masses, uncertainty, base_rates)
        - 每个张量的形状分别为 `[batch, n_states]`, `[batch]`, `[batch, n_states]`
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - (belief_masses, uncertainty, base_rates) - 融合后的意见

    Notes
    -----
    累积融合算子（⊕）在数学上对应于贝叶斯定理的更新，而在证据空间中，
    贝叶斯更新就表现为证据的简单加法。即 evidence(A ⊕ B) = evidence(A) + evidence(B)。
    """
    if not sources:
        raise ValueError('至少需要一个意见')

    if len(sources) == 1:
        return sources[0]

    # 获取基本信息
    belief_masses_0, uncertainty_0, base_rates_0 = sources[0]
    batch_size, n_states = belief_masses_0.shape
    device, dtype = belief_masses_0.device, belief_masses_0.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    # 检查维度一致性
    for i, (bm, u, br) in enumerate(sources):
        if bm.shape != (batch_size, n_states):
            raise ValueError(f'意见 {i} 的信念质量维度不一致')
        if u.shape != (batch_size,):
            raise ValueError(f'意见 {i} 的不确定性维度不一致')
        if br.shape != (batch_size, n_states):
            raise ValueError(f'意见 {i} 的基率维度不一致')

    # 检查基率一致性并计算平均基率
    base_rates_list = [br for _, _, br in sources]

    # 简单使用平均基率（可以考虑更复杂的一致性检查）
    avg_base_rates = torch.stack(base_rates_list, dim=0).mean(dim=0)  # [batch, n_states]

    # 累积证据
    total_evidence = torch.zeros(batch_size, n_states, device=device, dtype=dtype)
    for belief_masses, uncertainty, _ in sources:
        evidence = opinion_to_evidence(belief_masses, uncertainty, W)
        total_evidence += evidence

    # 转换回意见
    return evidence_to_opinion(total_evidence, avg_base_rates, W)


def averaging_fusion(
    sources: typing.Sequence[Tuple['Tensor', 'Tensor', 'Tensor']],
    W: Optional[float] = None,
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """平均融合 (Averaging Belief Fusion, ABF)：组合多个依赖信息源的意见

    Parameters
    ----------
    - `opinions` : `list[Tuple[Tensor, Tensor, Tensor]]`
        - 要融合的意见列表，每个意见为 (belief_masses, uncertainty, base_rates)
        - 每个张量的形状分别为 `[batch, n_states]`, `[batch]`, `[batch, n_states]`
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - (belief_masses, uncertainty, base_rates) - 融合后的意见
    """
    if not sources:
        raise ValueError('至少需要一个意见')

    if len(sources) == 1:
        return sources[0]

    n = len(sources)

    # 获取基本信息
    belief_masses_0, uncertainty_0, base_rates_0 = sources[0]
    batch_size, n_states = belief_masses_0.shape
    device, dtype = belief_masses_0.device, belief_masses_0.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    # 检查维度一致性
    for i, (bm, u, br) in enumerate(sources):
        if bm.shape != (batch_size, n_states):
            raise ValueError(f'意见 {i} 的信念质量维度不一致')
        if u.shape != (batch_size,):
            raise ValueError(f'意见 {i} 的不确定性维度不一致')
        if br.shape != (batch_size, n_states):
            raise ValueError(f'意见 {i} 的基率维度不一致')

    # 计算平均基率
    base_rates_list = [br for _, _, br in sources]
    avg_base_rates = torch.stack(base_rates_list, dim=0).mean(dim=0)  # [batch, n_states]

    # 平均证据
    total_evidence = torch.zeros(batch_size, n_states, device=device, dtype=dtype)
    for belief_masses, uncertainty, _ in sources:
        evidence = opinion_to_evidence(belief_masses, uncertainty, W)
        total_evidence += evidence

    avg_evidence = total_evidence / n

    # 转换回意见
    return evidence_to_opinion(avg_evidence, avg_base_rates, W)


def weighted_fusion(
    sources: typing.Sequence[Tuple['Tensor', 'Tensor', 'Tensor']],
    W: Optional[float] = None,
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """加权融合 (Weighted Belief Fusion, WBF)：基于置信度组合意见

    Parameters
    ----------
    - `opinions` : `list[Tuple[Tensor, Tensor, Tensor]]`
        - 要融合的意见列表，每个意见为 (belief_masses, uncertainty, base_rates)
        - 每个张量的形状分别为 `[batch, n_states]`, `[batch]`, `[batch, n_states]`
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - (belief_masses, uncertainty, base_rates) - 融合后的意见
    """
    if not sources:
        raise ValueError('至少需要一个意见')

    if len(sources) == 1:
        return sources[0]

    # 获取基本信息
    belief_masses_0, uncertainty_0, base_rates_0 = sources[0]
    batch_size, n_states = belief_masses_0.shape
    device, dtype = belief_masses_0.device, belief_masses_0.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    # 数值稳定性
    eps = 1e-12 if dtype == torch.float64 else 1e-7

    # 检查维度一致性
    for i, (bm, u, br) in enumerate(sources):
        if bm.shape != (batch_size, n_states):
            raise ValueError(f'意见 {i} 的信念质量维度不一致')
        if u.shape != (batch_size,):
            raise ValueError(f'意见 {i} 的不确定性维度不一致')
        if br.shape != (batch_size, n_states):
            raise ValueError(f'意见 {i} 的基率维度不一致')

    # 计算置信度加权和
    weighted_sum_evidence = torch.zeros(batch_size, n_states, device=device, dtype=dtype)
    weighted_sum_base_rates = torch.zeros(
        batch_size, n_states, device=device, dtype=dtype
    )
    total_confidence = torch.zeros(batch_size, device=device, dtype=dtype)

    for belief_masses, uncertainty, base_rates in sources:
        confidence = 1.0 - uncertainty  # [batch]

        # 只有在置信度大于零时，意见才有贡献
        valid_mask = confidence > eps
        if valid_mask.any():
            evidence = opinion_to_evidence(belief_masses, uncertainty, W)

            # 加权累积
            confidence_expanded = confidence.unsqueeze(1)  # [batch, 1]
            weighted_sum_evidence += evidence * confidence_expanded
            weighted_sum_base_rates += base_rates * confidence_expanded
            total_confidence += confidence

    # 处理所有意见都完全不确定的情况
    invalid_mask = total_confidence <= eps
    if invalid_mask.any():
        # 对于完全不确定的批次，使用平均基率
        base_rates_list = [br for _, _, br in sources]
        avg_base_rates = torch.stack(base_rates_list, dim=0).mean(
            dim=0
        )  # [batch, n_states]

        # 创建完全不确定的意见
        fused_belief_masses = torch.zeros(
            batch_size, n_states, device=device, dtype=dtype
        )
        fused_uncertainty = torch.ones(batch_size, device=device, dtype=dtype)
        fused_base_rates = avg_base_rates.expand(batch_size, -1)

        # 对于有效批次，计算加权平均
        if (~invalid_mask).any():
            valid_evidence = weighted_sum_evidence[~invalid_mask] / total_confidence[
                ~invalid_mask
            ].unsqueeze(1)
            valid_base_rates = weighted_sum_base_rates[~invalid_mask] / total_confidence[
                ~invalid_mask
            ].unsqueeze(1)

            valid_belief, valid_uncertainty, _ = evidence_to_opinion(
                valid_evidence, valid_base_rates, W
            )

            fused_belief_masses[~invalid_mask] = valid_belief
            fused_uncertainty[~invalid_mask] = valid_uncertainty
            fused_base_rates[~invalid_mask] = valid_base_rates

        return fused_belief_masses, fused_uncertainty, fused_base_rates
    else:
        # 所有批次都有效
        fused_evidence = weighted_sum_evidence / total_confidence.unsqueeze(1)
        fused_base_rates = weighted_sum_base_rates / total_confidence.unsqueeze(1)

        return evidence_to_opinion(fused_evidence, fused_base_rates, W)


# ========================= Trust Network ========================= #


def trust_discounting(
    trust_opinion: Tuple['Tensor', 'Tensor', 'Tensor'],
    source_opinion: Tuple['Tensor', 'Tensor', 'Tensor'],
    trust_state_idx: int = 0,
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """信任折扣：根据信任意见调整源意见

    该操作符用于根据对信息源的信任度来调整（折扣）其提供的意见。
    例如，在信任链 A -> B -> X 中，A 对 B 的信任意见 (trust_opinion) 会折扣
    B 对 X 的原始意见 (source_opinion)，从而得出 A 对 X 的派生意见。

    Parameters
    ----------
    - `trust_opinion` : `Tuple[Tensor, Tensor, Tensor]`
        - 信任意见 (belief_masses, uncertainty, base_rates)，必须是二元意见
        - 形状分别为 `[batch, 2]`, `[batch]`, `[batch, 2]`
    - `source_opinion` : `Tuple[Tensor, Tensor, Tensor]`
        - 源意见 (belief_masses, uncertainty, base_rates)，可以是多元意见
        - 形状分别为 `[batch, n_states]`, `[batch]`, `[batch, n_states]`
    - `trust_state_idx` : `int`
        - 信任意见状态的索引，默认为 0 (以第一个状态为 belief 状态)

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - 折扣后的意见 (belief_masses, uncertainty, base_rates)

    Raises
    ------
    ValueError
        当信任意见不是二元意见时
    """
    trust_belief_masses, trust_uncertainty, trust_base_rates = trust_opinion
    source_belief_masses, source_uncertainty, source_base_rates = source_opinion

    # 检查信任意见是否为二元意见
    if trust_belief_masses.shape[1] != 2:
        raise ValueError('信任意见 (trust_opinion) 必须是二元意见 (n_states=2)')

    # 检查批次维度一致性
    batch_size = trust_belief_masses.shape[0]
    if source_belief_masses.shape[0] != batch_size:
        raise ValueError('信任意见和源意见的批次维度必须一致')

    # 计算信任意见的投影概率作为折扣系数
    trust_projected_prob = projected_probability(
        trust_belief_masses,
        trust_uncertainty,
        trust_base_rates,
        belief_idx=trust_state_idx,
    )

    # 对源意见的信念质量进行折扣
    discount_coeff = trust_projected_prob.unsqueeze(1)  # [batch, 1]
    discounted_beliefs = discount_coeff * source_belief_masses  # [batch, n_states]

    # 计算新的不确定性
    discounted_uncertainty = 1.0 - torch.sum(discounted_beliefs, dim=1)  # [batch]

    # 基率继承自源意见
    derived_base_rates = source_base_rates

    return discounted_beliefs, discounted_uncertainty, derived_base_rates


def transitive_trust_discounting(
    trust_path: typing.Sequence[Tuple['Tensor', 'Tensor', 'Tensor']],
    source_opinion: Tuple['Tensor', 'Tensor', 'Tensor'],
    trust_state_idx: int = 0,
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """多级信任传递折扣

    处理一个长的信任链，例如 A1 -> A2 -> ... -> An -> X。
    最终的折扣系数是路径上所有信任意见投影概率的乘积。

    Parameters
    ----------
    - `trust_path` : `list[Tuple[Tensor, Tensor, Tensor]]`
        - 信任路径上的意见列表，每个意见都必须是二元意见
        - 例如 [ω_A2^A1, ω_A3^A2, ..., ω_An^A(n-1)]
    - `source_opinion` : `Tuple[Tensor, Tensor, Tensor]`
        - 路径末端的源意见 (belief_masses, uncertainty, base_rates)

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - 经过整个信任链折扣后的最终派生意见

    Raises
    ------
    ValueError
        当信任路径为空，或路径中有任何一个意见不是二元意见时
    """
    if not trust_path:
        raise ValueError('信任路径不能为空')

    source_belief_masses, source_uncertainty, source_base_rates = source_opinion
    batch_size = source_belief_masses.shape[0]
    device, dtype = source_belief_masses.device, source_belief_masses.dtype

    # 计算整个信任路径的总体折扣系数
    overall_discount_coefficient = torch.ones(batch_size, device=device, dtype=dtype)

    for i, trust_op in enumerate(trust_path):
        trust_belief_masses, trust_uncertainty, trust_base_rates = trust_op

        # 检查是否为二元意见
        if trust_belief_masses.shape[1] != 2:
            raise ValueError(f'信任路径中的第 {i} 个意见必须是二元意见')

        # 检查批次维度
        if trust_belief_masses.shape[0] != batch_size:
            raise ValueError(f'信任路径中的第 {i} 个意见的批次维度不一致')

        # 计算投影概率并累积折扣系数
        trust_prob = projected_probability(
            trust_belief_masses,
            trust_uncertainty,
            trust_base_rates,
            belief_idx=trust_state_idx,
        )
        overall_discount_coefficient *= trust_prob

    # 对源意见进行折扣
    discount_coeff = overall_discount_coefficient.unsqueeze(1)  # [batch, 1]
    discounted_beliefs = discount_coeff * source_belief_masses

    # 计算新的不确定性
    discounted_uncertainty = 1.0 - torch.sum(discounted_beliefs, dim=1)

    # 基率继承自源意见
    derived_base_rates = source_base_rates

    return discounted_beliefs, discounted_uncertainty, derived_base_rates


# ========================= Conflict Measures ========================= #


def degree_of_conflict(
    opinion1: Tuple['Tensor', 'Tensor', 'Tensor'],
    opinion2: Tuple['Tensor', 'Tensor', 'Tensor'],
) -> Tuple['Tensor', 'Tensor', 'Tensor']:
    """计算两个主观意见之间的冲突度

    冲突度是投影距离 (PD) 和联合确定性 (CC) 的乘积。

    Parameters
    ----------
    - `opinion1` : `Tuple[Tensor, Tensor, Tensor]`
        - 第一个意见 (belief_masses, uncertainty, base_rates)
    - `opinion2` : `Tuple[Tensor, Tensor, Tensor]`
        - 第二个意见 (belief_masses, uncertainty, base_rates)

    Returns
    -------
    - `Tuple[Tensor, Tensor, Tensor]`
        - (conflict, projected_distance, conjunctive_certainty) - 冲突度、投影距离、联合确定性

    Raises
    ------
    ValueError
        当两个意见的状态数不一致时

    Notes
    -----
    - **投影距离 (PD)**: 衡量两个意见的期望概率分布之间的差异，
      其值在 [0, 1] 范围内。PD = 0 表示期望概率完全一致。
      PD = sum(|P1_k - P2_k|) / 2

    - **联合确定性 (CC)**: 衡量两个意见共同的确定性程度，
      其值在 [0, 1] 范围内。CC = 0 表示至少有一个意见是完全不确定的。
      CC = (1 - u1) * (1 - u2)

    - **冲突度 (DC)**: 结合了距离和确定性。只有当两个意见都比较确定
      (CC 较高) 且期望概率相差很大 (PD 较高) 时，冲突度才会高。
      DC = PD * CC
    """
    belief_masses1, uncertainty1, base_rates1 = opinion1
    belief_masses2, uncertainty2, base_rates2 = opinion2

    # 验证输入维度一致性
    if belief_masses1.shape != belief_masses2.shape:
        raise ValueError('两个意见的信念质量维度必须一致')
    if uncertainty1.shape != uncertainty2.shape:
        raise ValueError('两个意见的不确定性维度必须一致')
    if base_rates1.shape != base_rates2.shape:
        raise ValueError('两个意见的基率维度必须一致')

    # 计算投影概率
    p1 = projected_probabilities(
        belief_masses1, uncertainty1, base_rates1
    )  # [batch, n_states]
    p2 = projected_probabilities(
        belief_masses2, uncertainty2, base_rates2
    )  # [batch, n_states]

    # 投影距离 (Projected Distance, PD)
    # PD(ω¹, ω²) = Σ|P¹_k - P²_k| / 2
    projected_distance = 0.5 * torch.sum(torch.abs(p1 - p2), dim=1)  # [batch]

    # 联合确定性 (Conjunctive Certainty, CC)
    # CC(ω¹, ω²) = (1 - u¹) * (1 - u²)
    conjunctive_certainty = (1.0 - uncertainty1) * (1.0 - uncertainty2)  # [batch]

    # 冲突度 (Degree of Conflict, DC)
    # DC(ω¹, ω²) = PD * CC
    conflict = projected_distance * conjunctive_certainty  # [batch]

    return conflict, projected_distance, conjunctive_certainty


def opinion_distance(
    opinion1: Tuple['Tensor', 'Tensor', 'Tensor'],
    opinion2: Tuple['Tensor', 'Tensor', 'Tensor'],
    metric: str = 'euclidean',
) -> 'Tensor':
    """计算两个意见之间的距离

    注意：这并非 Subjective Logic 中有理论定义的计算方式。
    如果要衡量两个意见之间的冲突程度，推荐使用 `degree_of_conflict`。

    Parameters
    ----------
    - `opinion1` : `Tuple[Tensor, Tensor, Tensor]`
        - 第一个意见 (belief_masses, uncertainty, base_rates)
    - `opinion2` : `Tuple[Tensor, Tensor, Tensor]`
        - 第二个意见 (belief_masses, uncertainty, base_rates)
    - `metric` : `str`
        - 距离度量方式，支持 'euclidean', 'manhattan', 'cosine'

    Returns
    -------
    - `Tensor`
        - 距离值，形状为 `[batch]`

    Raises
    ------
    ValueError
        当两个意见的状态数不一致，或不支持的距离度量时
    """
    belief_masses1, uncertainty1, _ = opinion1
    belief_masses2, uncertainty2, _ = opinion2

    # 验证维度一致性
    if belief_masses1.shape != belief_masses2.shape:
        raise ValueError('两个意见的状态数必须一致')
    if uncertainty1.shape != uncertainty2.shape:
        raise ValueError('两个意见的批次维度必须一致')

    # 构建完整向量 [beliefs, uncertainty]
    vec1 = torch.cat(
        [belief_masses1, uncertainty1.unsqueeze(1)], dim=1
    )  # [batch, n_states+1]
    vec2 = torch.cat(
        [belief_masses2, uncertainty2.unsqueeze(1)], dim=1
    )  # [batch, n_states+1]

    if metric == 'euclidean':
        return torch.sqrt(torch.sum((vec1 - vec2) ** 2, dim=1))
    elif metric == 'manhattan':
        return torch.sum(torch.abs(vec1 - vec2), dim=1)
    elif metric == 'cosine':
        # 余弦距离 = 1 - 余弦相似度
        dot_product = torch.sum(vec1 * vec2, dim=1)
        norm1 = torch.sqrt(torch.sum(vec1**2, dim=1))
        norm2 = torch.sqrt(torch.sum(vec2**2, dim=1))
        cosine_similarity = dot_product / (norm1 * norm2 + 1e-8)  # 数值稳定性
        return 1.0 - cosine_similarity
    else:
        raise ValueError(
            f'不支持的距离度量: {metric}。支持的选项: euclidean, manhattan, cosine'
        )


# ========================= DST Evidence Fusion ========================= #


def dempster_shafer_fusion(
    alpha_sources: typing.Sequence['Tensor'],
    conflict_handling: Literal['dempster', 'yager', 'zhang'] = 'dempster',
    base_rates: Optional['Tensor'] = None,
    W: Optional[float] = None,
) -> Tuple['Tensor', 'Tensor']:
    """使用 DST 证据理论融合多个狄利克雷参数向量

    基于 Dempster-Shafer 证据理论的组合规则来融合多个独立的证据源。
    该方法将 Dirichlet 参数转换为基本信念指派 (Basic Belief Assignment, BBA)，
    应用 DST 组合规则处理冲突，然后转换回 Dirichlet 参数。

    Parameters
    ----------
    - `alpha_sources` : `list[Tensor]`
        - 要融合的 Dirichlet 参数列表，每个张量形状为 `[batch, n_states]`
    - `conflict_handling` : `Literal['dempster', 'yager', 'zhang']`
        - 冲突处理策略：
        - 'dempster': 经典 Dempster 规则，归一化处理冲突
        - 'yager': Yager 规则，将冲突分配给无知状态
        - 'zhang': Zhang 规则，基于信任度重分配冲突
    - `base_rates` : `Tensor`, optional
        - 基率张量，形状为 `[n_states]` 或 `[batch, n_states]`，默认为均匀分布
    - `W` : `float`, optional
        - 非信息先验权重，默认为 n_states

    Returns
    -------
    - `Tuple[Tensor, Tensor]`
        - (fused_alpha, conflict_measure) - 融合后的 Dirichlet 参数和冲突度测量

    Raises
    ------
    ValueError
        当输入参数维度不一致或冲突处理策略不支持时

    Notes
    -----
    DST 融合过程：
    1. α → 证据 → 基本信念指派 (BBA)
    2. 应用 Dempster 组合规则：m(A) = Σ{m1(B)m2(C) : B∩C=A} / (1-K)
    3. 计算冲突度：K = Σ{m1(B)m2(C) : B∩C=∅}
    4. 根据策略处理冲突并归一化
    5. BBA → 证据 → α
    """
    if not alpha_sources:
        raise ValueError('至少需要一个 alpha 参数')

    if len(alpha_sources) == 1:
        return alpha_sources[0], torch.zeros(alpha_sources[0].shape[0])

    # 获取基本信息
    alpha_0 = alpha_sources[0]
    batch_size, n_states = alpha_0.shape
    device, dtype = alpha_0.device, alpha_0.dtype

    # 默认参数处理
    if W is None:
        W = float(n_states)

    if base_rates is None:
        base_rates = torch.ones(n_states, device=device, dtype=dtype) / n_states

    # 广播处理
    if base_rates.dim() == 1:
        base_rates = base_rates.unsqueeze(0).expand(batch_size, -1)

    # 检查维度一致性
    for i, alpha in enumerate(alpha_sources):
        if alpha.shape != (batch_size, n_states):
            raise ValueError(
                f'Alpha {i} 的维度不一致，期望 {(batch_size, n_states)}，得到 {alpha.shape}'
            )

    # 将 Dirichlet 参数转换为基本信念指派 (BBA)
    bba_sources = []
    for alpha in alpha_sources:
        # α → evidence → opinion → normalized BBA
        evidence = dirichlet_to_evidence(alpha, base_rates, W)
        belief_masses, uncertainty, _ = evidence_to_opinion(evidence, base_rates, W)

        # 构建 BBA：包含各状态的信念 + 不确定性 (无知状态)
        bba = torch.cat(
            [belief_masses, uncertainty.unsqueeze(1)], dim=1
        )  # [batch, n_states+1]
        bba_sources.append(bba)

    # 执行 DST 融合
    fused_bba, conflict = _dempster_combination_rule(
        bba_sources, conflict_handling, device, dtype
    )

    # 从融合后的 BBA 转换回 Dirichlet 参数
    # 分离信念质量和不确定性
    fused_belief_masses = fused_bba[:, :n_states]  # [batch, n_states]
    fused_uncertainty = fused_bba[:, n_states]  # [batch]

    # BBA → opinion → evidence → α
    fused_evidence = opinion_to_evidence(fused_belief_masses, fused_uncertainty, W)
    fused_alpha = evidence_to_dirichlet(fused_evidence, base_rates, W)

    return fused_alpha, conflict


def _dempster_combination_rule(
    bba_sources: typing.Sequence['Tensor'],
    conflict_handling: str,
    device: torch.device,
    dtype: torch.dtype,
) -> Tuple['Tensor', 'Tensor']:
    """执行 Dempster 组合规则的核心实现

    Parameters
    ----------
    - `bba_sources` : `list[Tensor]`
        - BBA 列表，每个张量形状为 `[batch, n_states+1]` (最后一维是无知状态)
    - `conflict_handling` : `str`
        - 冲突处理策略
    - `device` : `torch.device`
        - 设备类型
    - `dtype` : `torch.dtype`
        - 数据类型

    Returns
    -------
    - `Tuple[Tensor, Tensor]`
        - (fused_bba, conflict) - 融合后的BBA和冲突度
    """
    # 初始化为第一个 BBA
    fused_bba = bba_sources[0].clone()
    batch_size, n_focal = fused_bba.shape
    n_states = n_focal - 1  # 减去无知状态

    total_conflict = torch.zeros(batch_size, device=device, dtype=dtype)

    # 逐步融合剩余的 BBA
    for bba in bba_sources[1:]:
        fused_bba, conflict = _combine_two_bba(
            fused_bba, bba, conflict_handling, n_states
        )
        total_conflict += conflict

    return fused_bba, total_conflict


def _combine_two_bba(
    bba1: 'Tensor', bba2: 'Tensor', conflict_handling: str, n_states: int
) -> Tuple['Tensor', 'Tensor']:
    """组合两个 BBA 的核心算法

    实现 Dempster 组合规则：m(A) = Σ{m1(B)m2(C) : B∩C=A} / (1-K)

    Parameters
    ----------
    - `bba1`, `bba2` : `Tensor`
        - 两个 BBA，形状为 `[batch, n_states+1]`
    - `conflict_handling` : `str`
        - 冲突处理策略
    - `n_states` : `int`
        - 状态数量（不包括无知状态）

    Returns
    -------
    - `Tuple[Tensor, Tensor]`
        - (combined_bba, conflict) - 组合后的BBA和冲突度
    """
    dtype = bba1.dtype

    # 为简化实现，我们采用直接的矩阵运算方式
    # 这里实现简化版的 DST 组合规则

    # 分离各状态信念和无知状态
    m1_states = bba1[:, :n_states]  # [batch, n_states]
    m1_ignorance = bba1[:, n_states]  # [batch]
    m2_states = bba2[:, :n_states]  # [batch, n_states]
    m2_ignorance = bba2[:, n_states]  # [batch]

    # 计算状态-状态的组合（一致性支持）
    # m_combined[i] = m1[i] * m2_ignorance + m2[i] * m1_ignorance + m1[i] * m2[i]
    consistent_support = (
        m1_states * m2_ignorance.unsqueeze(1)  # m1状态 × m2无知
        + m2_states * m1_ignorance.unsqueeze(1)  # m2状态 × m1无知
        + m1_states * m2_states  # m1状态 × m2状态 (相同状态)
    )

    # 计算冲突（不同状态间的交集为空）
    # conflict = Σ_{i≠j} m1_i * m2_j
    # 高效的向量化计算方法:
    # Σ_{i,j} m1_i * m2_j = (Σ m1_i) * (Σ m2_j)
    # Σ_{i=j} m1_i * m2_j = Σ (m1_i * m2_i)
    # 因此, conflict = (Σ m1_i) * (Σ m2_j) - Σ (m1_i * m2_i)
    m1_sum = torch.sum(m1_states, dim=1)
    m2_sum = torch.sum(m2_states, dim=1)
    m1_dot_m2 = torch.sum(m1_states * m2_states, dim=1)
    conflict = m1_sum * m2_sum - m1_dot_m2

    # 计算新的无知状态质量
    combined_ignorance = m1_ignorance * m2_ignorance

    # 根据冲突处理策略处理结果
    if conflict_handling == 'dempster':
        # 经典 Dempster 规则：归一化，忽略冲突
        normalization = 1.0 - conflict
        # 数值稳定性处理
        eps = 1e-12 if dtype == torch.float64 else 1e-7
        normalization = torch.clamp(normalization, min=eps)

        combined_states = consistent_support / normalization.unsqueeze(1)
        combined_ignorance = combined_ignorance / normalization

    elif conflict_handling == 'yager':
        # Yager 规则：将冲突分配给无知状态
        combined_states = consistent_support
        combined_ignorance = combined_ignorance + conflict

    elif conflict_handling == 'zhang':
        # Zhang 规则：基于可信度重分配冲突
        # 简化实现：按比例重分配
        total_support = torch.sum(
            consistent_support, dim=1, keepdim=True
        ) + combined_ignorance.unsqueeze(1)
        conflict_redistribution = conflict.unsqueeze(1) * (
            consistent_support / (total_support + 1e-8)
        )

        combined_states = consistent_support + conflict_redistribution
        # 无知状态保持不变

    else:
        raise ValueError(f'不支持的冲突处理策略: {conflict_handling}')

    # 确保归一化
    combined_bba = torch.cat([combined_states, combined_ignorance.unsqueeze(1)], dim=1)
    bba_sum = torch.sum(combined_bba, dim=1, keepdim=True)
    combined_bba = combined_bba / (bba_sum + 1e-8)  # 数值稳定性

    return combined_bba, conflict



import math
from typing import Optional, cast

import torch
import torch.nn as nn
import torch.nn.functional as F
from typing_extensions import deprecated

from zyplib.nn.modules.base import BaseModule

from .attn_utils import combine_heads, split_heads


@deprecated('个人测试版本; 实际使用推荐 torch.nn.functional.scaled_dot_product_attention')
def my_scaled_dot_product_attention(
    q: torch.Tensor,
    k: torch.Tensor,
    v: torch.Tensor,
    mask: Optional[torch.Tensor] = None,
    dropout: Optional[nn.Dropout] = None,
) -> tuple[torch.Tensor, torch.Tensor]:
    """缩放点积注意力核心计算 (个人实现)

    Parameters
    ----------
    - `q` : `torch.Tensor`
        - 查询张量，形状: (batch_size, num_heads, seq_len_q, d_k)
    - `k` : `torch.Tensor`
        - 键张量，形状: (batch_size, num_heads, seq_len_k, d_k)
    - `v` : `torch.Tensor`
        - 值张量，形状: (batch_size, num_heads, seq_len_v, d_v)
    - `mask` : `torch.Tensor`, optional
        - 注意力掩码, by default None
    - `dropout` : `nn.Dropout`, optional
        - Dropout 层, by default None

    Returns
    ----------
    - `Tuple[torch.Tensor, torch.Tensor]`
        - (output, attention_weights)

    Examples
    --------
    ```python
    >>> import torch
    >>> from zyplib.nn.modules.attention.attn_utils import scaled_dot_product_attention
    >>>
    >>> q = torch.randn(32, 8, 100, 64)
    >>> k = torch.randn(32, 8, 100, 64)
    >>> v = torch.randn(32, 8, 100, 64)
    >>>
    >>> output, attn_weights = scaled_dot_product_attention(q, k, v)
    >>> print(output.shape)  # torch.Size([32, 8, 100, 64])
    >>> print(attn_weights.shape)  # torch.Size([32, 8, 100, 100])
    ```
    """
    d_k = q.size(-1)
    scores = torch.matmul(q, k.transpose(-2, -1)) / math.sqrt(d_k)

    if mask is not None:
        scores = scores.masked_fill(mask == 0, float('-inf'))

    attn_weights = F.softmax(scores, dim=-1)

    if dropout is not None:
        attn_weights = dropout(attn_weights)

    output = torch.matmul(attn_weights, v)
    return output, attn_weights


class ScaledDotProductAttention(BaseModule):
    def __init__(self, dropout: float = 0.0):
        """计算缩放点积注意力 (Scaled Dot-Product Attention)

        多头注意力机制的核心组件。

        Parameters
        ----------
        - `dropout` : `float`, optional
            - Dropout 比率, 默认为 0.0.

        """
        super().__init__()
        self.dropout = dropout

    def forward(
        self,
        q: torch.Tensor,
        k: torch.Tensor,
        v: torch.Tensor,
        mask: Optional[torch.Tensor] = None,
    ) -> tuple[torch.Tensor, torch.Tensor]:
        """
        缩放点积注意力的前向传播过程。

        Parameters
        ----------
        - `q` : `torch.Tensor`
            - 查询张量, 形状: `(batch_size, num_heads, seq_len_q, d_k)`
        - `k` : `torch.Tensor`
            - 键张量, 形状: `(batch_size, num_heads, seq_len_k, d_k)`
        - `v` : `torch.Tensor`
            - 值张量, 形状: `(batch_size, num_heads, seq_len_v, d_v)`
        - `mask` : `torch.Tensor`, optional
            - 掩码张量, 形状: `(batch_size, 1, seq_len_q, seq_len_k)`, 默认为 None.

        Returns
        -------
        - **output** (`torch.Tensor`): 应用注意力后的上下文向量, 形状: `(batch_size, num_heads, seq_len_q, d_v)`.
        """
        # dropout_layer = (
        #     nn.Dropout(self.dropout) if self.dropout > 0.0 and self.training else None
        # )
        # return my_scaled_dot_product_attention(q, k, v, mask, dropout_layer)
        # my_scaled 实现的 attention 效率更低, 使用官方实现的 scaled_dot_product_attention
        # 注意：F.scaled_dot_product_attention 只返回输出，不返回注意力权重
        dp = self.dropout if self.training else 0.0
        return cast(
            tuple[torch.Tensor, torch.Tensor],
            F.scaled_dot_product_attention(q, k, v, mask, dp),
        )


@deprecated('个人测试版本; 实际使用推荐 torch.nn.MultiheadAttention')
class MultiHeadAttention(BaseModule):
    def __init__(
        self, embed_dim: int, num_heads: int, dropout: float = 0.0, bias: bool = True
    ):
        """多头注意力 (Multi-Head Attention) 模块。

        该模块实现了 "Attention Is All You Need" 中描述的多头注意力机制。
        它允许模型在不同位置共同关注来自不同表示子空间的信息。

        Parameters
        ----------
        - `embed_dim` : `int`
            - 模型的总维度 (嵌入大小)。
        - `num_heads` : `int`
            - 并行注意力头的数量。`embed_dim` 必须能被 `num_heads` 整除。
        - `dropout` : `float`, optional
            - 注意力权重的 Dropout 比率, 默认为 0.0。

        Raises
        ------
        - `ValueError`
            - 如果 `embed_dim` 不能被 `num_heads` 整除。

        Examples
        --------
        ```python
        >>> import torch
        >>> from zyplib.nn.modules.attention.multihead import MultiHeadAttention
        >>>
        >>> # 自注意力示例
        >>> embed_dim, num_heads = 128, 8
        >>> seq_len, batch_size = 10, 32
        >>> x = torch.randn(batch_size, seq_len, embed_dim)
        >>>
        >>> mha = MultiHeadAttention(embed_dim, num_heads)
        >>> output, attn_weights = mha(x, x, x)
        >>>
        >>> print(f"输入形状: {x.shape}")
        >>> print(f"输出形状: {output.shape}")
        >>> print(f"注意力权重形状: {attn_weights.shape}")
        输入形状: torch.Size([32, 10, 128])
        输出形状: torch.Size([32, 10, 128])
        注意力权重形状: torch.Size([32, 8, 10, 10])
        ```
        """
        super().__init__()
        if embed_dim % num_heads != 0:
            raise ValueError(
                f'embed_dim ({embed_dim}) 必须能被 num_heads ({num_heads}) 整除'
            )

        self.embed_dim = embed_dim
        self.num_heads = num_heads
        self.d_k = embed_dim // num_heads

        self.W_q = nn.Linear(embed_dim, embed_dim, bias=bias)
        self.W_k = nn.Linear(embed_dim, embed_dim, bias=bias)
        self.W_v = nn.Linear(embed_dim, embed_dim, bias=bias)

        self.attention = ScaledDotProductAttention(dropout=dropout)

        self.fc = nn.Linear(embed_dim, embed_dim)

    def _split_heads(self, x: torch.Tensor) -> torch.Tensor:
        """将最后一个维度拆分为 (num_heads, d_k)。"""
        return split_heads(x, self.num_heads, self.d_k)

    def _combine_heads(self, x: torch.Tensor) -> torch.Tensor:
        """将多个头合并回原始形状。"""
        return combine_heads(x, self.embed_dim)

    def forward(
        self,
        q: torch.Tensor,
        k: torch.Tensor,
        v: torch.Tensor,
        mask: Optional[torch.Tensor] = None,
    ) -> tuple[torch.Tensor, torch.Tensor]:
        """
        多头注意力的前向传播过程。

        Parameters
        ----------
        - `q` : `torch.Tensor`
            - 查询张量, 形状: `(batch_size, seq_len_q, embed_dim)`.
        - `k` : `torch.Tensor`
            - 键张量, 形状: `(batch_size, seq_len_k, embed_dim)`.
        - `v` : `torch.Tensor`
            - 值张量, 形状: `(batch_size, seq_len_v, embed_dim)`.
        - `mask` : `torch.Tensor`, optional
            - 掩码张量。其形状取决于掩码类型:
              - 填充掩码 (padding mask): `(batch_size, 1, 1, seq_len_k)`
              - 序列掩码 (sequence mask): `(batch_size, 1, seq_len_q, seq_len_k)`
              默认为 None.

        Returns
        -------
        - `tuple[torch.Tensor, torch.Tensor]`
            - 一个元组，包含:
                - **output** (`torch.Tensor`): 最终的输出张量, 形状: `(batch_size, seq_len_q, embed_dim)`.
                - **attn_weights** (`torch.Tensor`): 注意力权重, 形状: `(batch_size, num_heads, seq_len_q, seq_len_k)`.
        """
        # 1. 线性投影
        q, k, v = self.W_q(q), self.W_k(k), self.W_v(v)

        # 2. 拆分为多个头
        q = self._split_heads(q)
        k = self._split_heads(k)
        v = self._split_heads(v)

        # 3. 缩放点积注意力
        # context, attn_weights = self.attention(q, k, v, mask=mask)
        context = self.attention(q, k, v, mask=mask)

        # 4. 合并头并通过最终的线性层
        context = self._combine_heads(context)
        output = self.fc(context)

        return output


class SelfAttention(BaseModule):
    def __init__(
        self,
        embed_dim: int,
        num_heads: int,
        dropout: float = 0.0,
        bias: bool = True,
        **kwargs,
    ):
        """自注意力 (Self-Attention) 模块。

        MultiHeadAttention 的特化版，其中 query, key, 和 value
        都来自同一个输入序列。

        Parameters
        ----------
        - `embed_dim` : `int`
            - 模型的总维度 (嵌入大小)。
        - `num_heads` : `int`
            - 并行注意力头的数量。`embed_dim` 必须能被 `num_heads` 整除。
        - `dropout` : `float`, optional
            - 注意力权重的 Dropout 比率, 默认为 0.0。

        Examples
        --------
        ```python
        >>> import torch
        >>> from zyplib.nn.modules.attention.multihead import SelfAttention
        >>>
        >>> embed_dim, num_heads = 128, 8
        >>> seq_len, batch_size = 10, 32
        >>> x = torch.randn(batch_size, seq_len, embed_dim)
        >>>
        >>> self_attn = SelfAttention(embed_dim, num_heads)
        >>> output, attn_weights = self_attn(x)
        >>>
        >>> print(f"输入形状: {x.shape}")
        >>> print(f"输出形状: {output.shape}")
        >>> print(f"注意力权重形状: {attn_weights.shape}")
        输入形状: torch.Size([32, 10, 128])
        输出形状: torch.Size([32, 10, 128])
        注意力权重形状: torch.Size([32, 8, 10, 10])
        ```
        """
        super().__init__()
        self.embed_dim = embed_dim
        self.num_heads = num_heads

        self.mha = nn.MultiheadAttention(
            embed_dim=embed_dim,
            num_heads=num_heads,
            dropout=dropout,
            bias=bias,
            batch_first=True,  # 确保输入输出形状为 [N, L, E]
            **kwargs,
        )

    def forward(
        self,
        x: torch.Tensor,
        attn_mask: Optional[torch.Tensor] = None,
        key_padding_mask: Optional[torch.Tensor] = None,
        need_weights=True,
        average_attn_weights=True,
    ) -> tuple[torch.Tensor, torch.Tensor]:
        """
        自注意力的前向传播过程。

        Parameters
        ----------
        - `x` : `torch.Tensor`
            - 输入张量, 形状: `(batch_size, seq_len, embed_dim)`.
        - `attn_mask` : `torch.Tensor`, optional
            - 2D 或 3D 掩码，防止 token 关注某些其他 token。
              形状: `(seq_len, seq_len)` 或 `(batch_size * num_heads, seq_len, seq_len)`.
              布尔类型中，True 表示“不被允许关注”。
        - `key_padding_mask` : `torch.Tensor`, optional
            - 布尔掩码，指示哪些 key token 是 padding，不应被关注。
              形状: `(batch_size, seq_len)`. True 表示该位置是 padding。

        Returns
        -------
        - `tuple[torch.Tensor, torch.Tensor]`
            - **output** (`torch.Tensor`): 输出张量, 形状: `(batch_size, seq_len, embed_dim)`.
            - **attn_weights** (`torch.Tensor`): 注意力权重
                - 如果 `average_attn_weights=False`, 则返
        """
        # 对于自注意力, query, key, value 都是 x
        # need_weights=True 确保返回注意力权重
        return self.mha(
            x,
            x,
            x,
            attn_mask=attn_mask,
            key_padding_mask=key_padding_mask,
            need_weights=need_weights,
            average_attn_weights=average_attn_weights,
        )


class CrossAttention(BaseModule):
    def __init__(
        self,
        embed_dim: int,
        num_heads: int,
        dropout: float = 0.0,
        bias: bool = True,
        **kwargs,
    ):
        """交叉注意力 (Cross-Attention) 模块。

        这是 MultiHeadAttention 的特化版本，其中 query 来自一个序列
        (例如，解码器输入)，而 key/value 来自另一个序列 (例如，编码器输出)。
        它通常用于编码器-解码器架构中。

        Parameters
        ----------
        - `embed_dim` : `int`
            - 模型的总维度 (嵌入大小)。
        - `num_heads` : `int`
            - 并行注意力头的数量。`embed_dim` 必须能被 `num_heads` 整除。
        - `dropout` : `float`, optional
            - 注意力权重的 Dropout 比率, 默认为 0.0。

        Examples
        --------
        ```python
        >>> import torch
        >>> from zyplib.nn.modules.attention.multihead import CrossAttention
        >>>
        >>> embed_dim, num_heads = 128, 8
        >>> seq_len_q, seq_len_kv, batch_size = 10, 15, 32
        >>> query = torch.randn(batch_size, seq_len_q, embed_dim)
        >>> context = torch.randn(batch_size, seq_len_kv, embed_dim)
        >>>
        >>> cross_attn = CrossAttention(embed_dim, num_heads)
        >>> output, attn_weights = cross_attn(query, context)
        >>>
        >>> print(f"Query 形状: {query.shape}")
        >>> print(f"Context 形状: {context.shape}")
        >>> print(f"输出形状: {output.shape}")
        >>> print(f"注意力权重形状: {attn_weights.shape}")
        Query 形状: torch.Size([32, 10, 128])
        Context 形状: torch.Size([32, 15, 128])
        输出形状: torch.Size([32, 10, 128])
        注意力权重形状: torch.Size([32, 8, 10, 15])
        ```
        """
        super().__init__()
        self.embed_dim = embed_dim
        self.num_heads = num_heads
        self.mha = nn.MultiheadAttention(
            embed_dim=embed_dim,
            num_heads=num_heads,
            dropout=dropout,
            bias=bias,
            batch_first=True,  # 确保输入输出形状为 [N, L, E]
            **kwargs,
        )

    def forward(
        self,
        query: torch.Tensor,
        context: torch.Tensor,
        attn_mask: Optional[torch.Tensor] = None,
        key_padding_mask: Optional[torch.Tensor] = None,
        need_weights=True,
        average_attn_weights=True,
    ) -> tuple[torch.Tensor, torch.Tensor]:
        """
        交叉注意力的前向传播过程。

        Parameters
        ----------
        - `query` : `torch.Tensor`
            - 查询张量, 形状: `(batch_size, seq_len_q, embed_dim)`.
        - `context` : `torch.Tensor`
            - 上下文张量 (用于 key 和 value), 形状: `(batch_size, seq_len_kv, embed_dim)`.
        - `attn_mask` : `torch.Tensor`, optional
            - 2D 掩码，防止 query token 关注某些 key token。
              形状: `(seq_len_q, seq_len_kv)`.
        - `key_padding_mask` : `torch.Tensor`, optional
            - 布尔掩码，指示哪些 key token 是 padding，不应被关注。
              形状: `(batch_size, seq_len_kv)`.

        Returns
        -------
        - `tuple[torch.Tensor, s=Fals
        """
        # 对于交叉注意力, key 和 value 都来自 context
        # average_attn_weights=False 返回每个头的注意力权重而不是平均值
        return self.mha(
            query,
            context,
            context,
            attn_mask=attn_mask,
            key_padding_mask=key_padding_mask,
            need_weights=need_weights,
            average_attn_weights=average_attn_weights,
        )



import typing

# Data augmentation for EEG signals - PyTorch only version
from typing import Literal, Optional, Tuple, cast, overload

import numpy as np
import torch

from zyplib.signal.filter import notch
from zyplib.utils.ensure import ensure_3dims

__all__ = [
    'sign_flip',
    'time_reverse',
    'add_white_noise',
    'scale_amplitude',
    'add_spike',
    'bandstop_aug',
    'ft_surrogate',
]


def _apply_notch(
    X: np.ndarray, srate: float, f0: float, Q: float, axis: int
) -> np.ndarray:
    return cast(np.ndarray, notch(X, srate, f0=f0, Q=Q, axis=axis))


def _use_random_state(random_state=None, device=None) -> torch.Generator:
    """检查和转换随机状态对象

    为 PyTorch 提供统一的随机数生成接口

    Parameters
    ----------
    - `random_state` : `None | int | torch.Generator`, optional
        随机状态或种子，默认为 None
    - `device` : `str | torch.device`, optional
        PyTorch 设备，用于生成设备感知的随机数，默认为 None

    Returns
    -------
    - `torch.Generator`
        随机状态对象
    """
    if isinstance(random_state, torch.Generator):
        return random_state
    elif random_state is None:
        return torch.Generator(device=device)
    elif isinstance(random_state, int):
        generator = torch.Generator(device=device)
        generator.manual_seed(random_state)
        return generator
    else:
        raise TypeError(f'Unsupported random_state type: {type(random_state)}')


def sign_flip(X: torch.Tensor) -> torch.Tensor:
    """上下颠倒信号

    将信号进行垂直翻转（乘以-1）。

    Parameters
    ----------
    - `X` : `torch.Tensor`
        - 输入数据，形状为 (C, T) 或 (N, C, T)

    Returns
    ----------
    - `torch.Tensor`
        - 翻转后的数据，形状与输入相同
    """
    return -X


def time_reverse(X: torch.Tensor) -> torch.Tensor:
    """时间反转

    将信号在时间维度上反转。

    Parameters
    ----------
    - `X` : `torch.Tensor`
        - 输入数据，形状为 (C, T) 或 (N, C, T)

    Returns
    ----------
    - `torch.Tensor`
        - 时间反转后的数据，形状与输入相同
    """
    return torch.flip(X, dims=[-1])


def add_white_noise(
    X: torch.Tensor,
    scale: float = 0.1,
    rel_power: bool = False,
    random_state: Optional[int] = None,
) -> torch.Tensor:
    """添加白噪声

    向信号中添加高斯白噪声。提供两种噪声生成方式：
    1. 绝对噪声强度（rel_power=False）：生成指定标准差的噪声
    2. 相对噪声强度（rel_power=True）：生成相对于原始信号功率的噪声

    Parameters
    ----------
    - `X` : `torch.Tensor`
        - 输入数据，形状为 (C, T) 或 (N, C, T)
    - `std` : `float`, optional
        - 噪声的标准差或相对功率，by default 0.1
        - 当 rel_power=False 时，表示噪声的绝对标准差
        - 当 rel_power=True 时，表示相对于信号功率的比例
    - `rel_power` : `bool`, optional
        - 是否使用相对功率模式，by default False
    - `random_state` : `Optional[int]`, optional
        - 随机种子，用于生成可重复的噪声，by default None

    Returns
    ----------
    - `torch.Tensor`
        - 添加噪声后的数据，形状与输入相同
    """
    device = X.device
    generator = _use_random_state(random_state, device=device)

    if rel_power:
        signal_power = torch.var(X, dim=-1, keepdim=True)
        noise_std = torch.sqrt(signal_power * scale)
        noise = (
            torch.randn(X.shape, generator=generator, device=device, dtype=X.dtype)
            * noise_std
        )
    else:
        noise = (
            torch.randn(X.shape, generator=generator, device=device, dtype=X.dtype)
            * scale
        )

    return X + noise


def scale_amplitude(
    X: torch.Tensor,
    scale_range: Tuple[float, float] = (0.9, 1.1),
    granularity: Literal['global', 'per_epoch', 'per_channel'] = 'per_channel',
    random_state: Optional[int] = None,
) -> torch.Tensor:
    """
    随机缩放信号的振幅。

    默认的行为是，随机扰动各个通道的幅值，在 [0.9, 1.1] 范围内。

    Parameters
    ----------
    - `X` : `torch.Tensor`
        - 输入数据，形状为 (C, T) 或 (N, C, T)。
    - `scale_range` : `Tuple[float, float]`, optional
        - 缩放因子的随机采样范围, by default (0.8, 1.2)。
    - `granularity` : `Literal['global', 'per_epoch', 'per_channel']`, optional
        - 缩放因子的作用粒度, by default 'per_channel'。
        - 'global': 整个输入使用一个随机缩放因子。
        - 'per_epoch': 每个 epoch (试验) 使用一个独立的随机因子。
        - 'per_channel': 每个通道使用一个独立的随机因子。
    - `random_state` : `Optional[int]`, optional
        - 随机种子，用于生成可复现的缩放因子, by default None。

    Returns
    -------
    - `torch.Tensor`
        - 振幅缩放后的数据，形状与输入相同。
    """
    device = X.device
    generator = _use_random_state(random_state, device=device)
    original_shape = X.shape
    X_proc = cast(torch.Tensor, ensure_3dims(X, newaxis='batch'))  # 返回 (N, C, T)

    N, C, T = X_proc.shape

    # 1. 根据清晰的粒度参数确定缩放因子的形状
    if granularity == 'global':
        shape = (1,)
    elif granularity == 'per_epoch':
        shape = (N,)
    elif granularity == 'per_channel':
        shape = (N, C)
    else:
        raise ValueError(f'不支持的粒度: {granularity}')

    # 2. 生成随机缩放因子
    scale_factor = torch.rand(shape, generator=generator, device=device, dtype=X.dtype)
    scale_factor = scale_factor * (scale_range[1] - scale_range[0]) + scale_range[0]

    # 3. 调整形状以进行广播
    while scale_factor.ndim < X_proc.ndim:
        scale_factor = scale_factor.unsqueeze(-1)

    # 4. 应用缩放并返回正确形状
    return (X_proc * scale_factor).reshape(original_shape)


def _make_gaussian_spike(
    t_width: float,
    srate: int,
    amplitude: float,
    *,
    device: Optional[torch.device] = None,
    dtype: Optional[torch.dtype] = None,
) -> torch.Tensor:
    """生成尖峰波形

    使用高斯包络生成持续时间极短的尖峰信号。

    Parameters
    ----------
    - `t_width` : `float`
        - 尖峰宽度（秒）
    - `srate` : `int`
        - 采样率（Hz）
    - `amplitude` : `float`
        - 尖峰峰值振幅，可正可负
    - `device` : `torch.device`, optional
        - PyTorch 设备
    - `dtype` : `torch.dtype`, optional
        - 数据类型

    Returns
    ----------
    - `torch.Tensor`
        - 长度为 round(t_width * srate) 的 1-D 波形
    """
    # Ensure the spike occupies at least one sample
    length = max(1, int(round(t_width * srate)))

    # Prefer an odd length so that the spike is centred
    if length % 2 == 0:
        length += 1

    t = torch.arange(length, device=device) - length // 2
    sigma = length / 6.0  # ≈99.7% of Gaussian within ±3σ (here ±length/2)
    spike = amplitude * torch.exp(-0.5 * (t / sigma) ** 2)
    if dtype is not None:
        spike = spike.to(dtype=dtype)
    return spike


@overload
def add_spike(
    X: torch.Tensor,
    srate: int,
    number: int,
    t_width: float,
    *,
    amplitude_mode: Literal[
        'fixed', 'random_absolute', 'random_relative', 'relative'
    ] = 'random_relative',
    amplitude: Tuple[float, float] | float = (-1.5, 1.5),
    position_granularity: Literal['per_epoch', 'per_channel'] = 'per_epoch',
    random_state: Optional[int] = None,
    return_spike_pos: Literal[False] = False,
) -> torch.Tensor: ...


@overload
def add_spike(
    X: torch.Tensor,
    srate: int,
    number: int,
    t_width: float,
    *,
    amplitude_mode: Literal[
        'fixed', 'random_absolute', 'random_relative', 'relative'
    ] = 'random_relative',
    amplitude: Tuple[float, float] | float = (-1.5, 1.5),
    position_granularity: Literal['per_epoch', 'per_channel'] = 'per_epoch',
    random_state: Optional[int] = None,
    return_spike_pos: Literal[True],
) -> tuple[torch.Tensor, torch.Tensor]: ...


def add_spike(
    X: torch.Tensor,
    srate: int,
    number: int,
    t_width: float,
    *,
    amplitude_mode: Literal[
        'fixed', 'random_absolute', 'random_relative', 'relative'
    ] = 'random_relative',
    amplitude: Tuple[float, float] | float = (-1.5, 1.5),
    position_granularity: Literal['per_epoch', 'per_channel'] = 'per_epoch',
    random_state: Optional[int] = None,
    return_spike_pos: bool = False,
) -> torch.Tensor | tuple[torch.Tensor, torch.Tensor]:
    """
    添加尖峰噪声。

    Parameters
    ----------
    - `X` : `torch.Tensor`
        - 输入数据，形状为 (C, T) 或 (N, C, T)。
    - `srate` : `int`
        - 采样率（Hz）。
    - `number` : `int`
        - 每个粒度单位（epoch或channel）插入的尖峰数量。
    - `t_width` : `float`
        - 单个尖峰的持续时间（秒）。
    - `amplitude_mode` : `Literal['fixed', 'random_absolute', 'random_relative', 'relative']`, optional
        - 振幅生成模式, by default 'random_relative'。
        - 'fixed': 使用固定的绝对振幅。
        - 'random_absolute': 从范围内随机采样一个绝对振幅。
        - 'random_relative': 从范围内随机采样一个比例因子，乘以通道标准差作为振幅。
        - 'relative': 使用固定的相对比例，乘以通道标准差作为振幅。
        - **注意**: 所有的相对模式都是相对于所在的通道，而非完整的 epoch 信号的标准差。
    - `amplitude` : `Tuple[float, float] | float`, optional
        - 根据 amplitude_mode 提供的值, by default (-1.5, 1.5)。
        - 如果 mode='fixed' 或 'relative', 应提供 float。
        - 如果 mode 是 'random_*', 应提供一个 (min, max) 的 tuple。
    - `position_granularity` : `Literal['per_epoch', 'per_channel']`, optional
        - 尖峰位置的生成粒度, by default 'per_epoch'。
        - 'per_epoch': 在每个 epoch 内生成一组共享的位置，应用到所有通道。
        - 'per_channel': 为每个通道独立生成一组位置。
    - `random_state` : `Optional[int]`, optional
        - 随机种子，保证可复现
    - `return_spike_pos` : `bool`, optional
        - 是否返回尖峰位置，by default False。

    Returns
    -------
    - `torch.Tensor`
        - 添加尖峰后的数据，形状与输入相同。
    """
    device = X.device
    dtype = X.dtype
    generator = _use_random_state(random_state, device=device)

    original_shape = X.shape
    X_proc = cast(torch.Tensor, ensure_3dims(X.clone(), newaxis='batch'))
    N, C, T = X_proc.shape

    actual_spike_len = len(_make_gaussian_spike(t_width, srate, 1.0, device=device))
    if T < actual_spike_len:
        raise ValueError(f'信号长度 {T} 小于尖峰长度 {actual_spike_len}')

    def _get_amplitude(batch_idx: int, ch_idx: int) -> float:
        if amplitude_mode == 'fixed':
            if not isinstance(amplitude, (float, int)):
                raise TypeError("For 'fixed' mode, amplitude must be a float or int.")
            return float(amplitude)
        elif amplitude_mode == 'relative':
            if not isinstance(amplitude, (float, int)):
                raise TypeError("For 'relative' mode, amplitude must be a float or int.")
            return float(amplitude) * torch.std(X_proc[batch_idx, ch_idx]).item()

        if not isinstance(amplitude, tuple) or len(amplitude) != 2:
            raise TypeError('For random modes, amplitude must be a (min, max) tuple.')

        rand_val = torch.rand(1, generator=generator, device=device).item()
        rand_val = rand_val * (amplitude[1] - amplitude[0]) + amplitude[0]

        if amplitude_mode == 'random_absolute':
            return rand_val
        elif amplitude_mode == 'random_relative':
            return rand_val * torch.std(X_proc[batch_idx, ch_idx]).item()
        else:
            raise ValueError(f'Unknown amplitude_mode: {amplitude_mode}')

    def _get_random_positions() -> torch.Tensor:
        if number <= (T - actual_spike_len):
            # 使用 torch.randperm 来避免重复
            positions = torch.randperm(
                T - actual_spike_len, generator=generator, device=device
            )[:number]
        else:
            positions = torch.randint(
                0, T - actual_spike_len, (number,), generator=generator, device=device
            )
        return positions

    for n in range(N):
        if position_granularity == 'per_channel':
            for c in range(C):
                positions = _get_random_positions()
                for pos in positions:
                    pos_int = int(pos.item())
                    amp = _get_amplitude(n, c)
                    spike = cast(
                        torch.Tensor,
                        _make_gaussian_spike(
                            t_width, srate, amp, device=device, dtype=dtype
                        ),
                    )
                    end_pos = pos_int + len(spike)
                    X_proc[n, c, pos_int:end_pos] += spike
        elif position_granularity == 'per_epoch':
            # 先为整个 epoch 生成共享的位置
            positions = _get_random_positions()
            for pos in positions:
                pos_int = int(pos.item())
                # 振幅仍然是每个通道独立计算的，这通常是期望的行为
                for c in range(C):
                    amp = _get_amplitude(n, c)
                    spike = cast(
                        torch.Tensor,
                        _make_gaussian_spike(
                            t_width, srate, amp, device=device, dtype=dtype
                        ),
                    )
                    end_pos = pos_int + len(spike)
                    X_proc[n, c, pos_int:end_pos] += spike
        else:
            raise ValueError(f'不支持的粒度: {position_granularity}')

    if return_spike_pos:
        return X_proc.reshape(original_shape), cast(torch.Tensor, positions / srate)
    return X_proc.reshape(original_shape)


def bandstop_aug(
    X: torch.Tensor,
    srate: int,
    bandwidth: float = 2,
    freqs_to_notch: Optional[int | typing.Sequence[float]] = None,
    random_state: Optional[int] = None,
    random_freq_range: Optional[Tuple[float, float]] = None,
    axis=-1,
) -> torch.Tensor:
    """
    带阻滤波器数据增强

    使用陷波滤波器（notch filter）对信号中的特定频率进行抑制，常用于去除工频干扰等。
    对每个目标频率，带阻宽度保持恒定。

    Parameters
    ----------
    - `X` : `torch.Tensor`
        - 输入数据，形状为 (C, T) 或 (N, C, T)
    - `srate` : `int`
        - 采样率（Hz）
    - `bandwidth` : `float`, optional
        - 带阻宽度（Hz），by default 2
        - 定义为每个频率点两侧的衰减带宽
    - `freqs_to_notch` : `Optional[list[float] | int]`, optional
        - 需要抑制的频率列表（Hz），by default [50]
        - 可指定多个频率，如 [50, 60] 同时抑制 50Hz 和 60Hz
        - 指定 int, 随机选择频带
    - `random_state` : `Optional[int]`, optional
        - 随机种子，用于生成可重复的随机数
    - `random_freq_range` : `Optional[Tuple[float, float]]`, optional
        - 随机选择频带的频率范围
    - `axis` : `int`, optional
        - 应用滤波的轴，by default -1

    Returns
    ----------
    - `torch.Tensor`
        - 滤波后的数据，形状与输入相同
    """
    if bandwidth <= 0:
        return X

    # 修复可变默认参数
    if freqs_to_notch is None:
        notch_freqs: typing.Sequence[float] = [50.0]
    elif isinstance(freqs_to_notch, int):
        freq_range = random_freq_range or (0.5, srate / 2 - 1)  # 浮点数上限

        # PyTorch 随机数生成
        generator = _use_random_state(random_state, device=X.device)
        sampled_freqs = torch.rand(freqs_to_notch, generator=generator, device=X.device)
        sampled_freqs = sampled_freqs * (freq_range[1] - freq_range[0]) + freq_range[0]
        sampled_freqs = torch.clamp(sampled_freqs, 0.1, srate / 2 - 0.1)
        notch_freqs = cast(list[float], sampled_freqs.cpu().numpy().tolist())
    else:
        notch_freqs = freqs_to_notch

    # PyTorch + NumPy 混合模式处理
    device = X.device
    dtype = X.dtype

    # 转换为 NumPy 进行滤波，确保连续内存布局
    X_np = X.detach().cpu().numpy()
    if not X_np.flags['C_CONTIGUOUS']:
        X_np = X_np.copy()

    # 对每个频率点分别应用陷波滤波
    for freq in notch_freqs:
        # 为每个频率正确计算 Q 值: Q = f0 / bandwidth
        Q = freq / bandwidth
        X_filtered = _apply_notch(X_np, srate, f0=freq, Q=Q, axis=axis)
        # 确保结果是连续的
        if not X_filtered.flags['C_CONTIGUOUS']:
            X_filtered = X_filtered.copy()
        X_np = X_filtered

    # 转换回 PyTorch
    return torch.from_numpy(X_np).to(dtype=dtype, device=device)


def _construct_symmetric_phase(
    positive_freq_phases: torch.Tensor, n_samples: int
) -> torch.Tensor:
    """
    根据给定的正频率相位，构建一个完整的共轭对称相位数组。

    Parameters
    ----------
    - positive_freq_phases : torch.Tensor
        一个复数数组，代表正频率部分的相位 (例如，由 j*φ 构成)。
        形状应为 (batch_size, channels, n_positive_freqs)。
    - n_samples : int
        原始信号的时间点总数。

    Returns
    -------
    - torch.Tensor
        完整的、具有共轭对称性的相位数组。
    """
    device = positive_freq_phases.device
    dtype = positive_freq_phases.dtype

    # 直流分量 (k=0) 的相位为0
    zeros = torch.zeros(
        (positive_freq_phases.shape[0], positive_freq_phases.shape[1], 1),
        dtype=dtype,
        device=device,
    )

    if n_samples % 2 == 0:  # 偶数长度
        # 奈奎斯特频率 (k=N/2) 的相位也为0
        full_phase = torch.cat(
            [
                zeros,
                positive_freq_phases,
                zeros,
                -torch.flip(positive_freq_phases, dims=[-1]),
            ],
            dim=-1,
        )
    else:  # 奇数长度
        full_phase = torch.cat(
            [zeros, positive_freq_phases, -torch.flip(positive_freq_phases, dims=[-1])],
            dim=-1,
        )
    return full_phase


def ft_surrogate(
    X: torch.Tensor,
    phase_noise_magnitude: float = 1.0,
    random_state=None,
    phase: torch.Tensor | typing.Sequence[float] | float | None = None,
) -> torch.Tensor:
    """FT surrogate 数据增强

    通过在频域中添加相位扰动来生成时间序列数据的替代样本，同时保留原始数据的功率谱。
    允许用户手动指定相位，或使用随机相位。

    Parameters
    ----------
    - `X` : `torch.Tensor`
        EEG 输入样本或批次，形状为 (C, T) 或 (N, C, T)
    - `phase_noise_magnitude` : `float`, optional
        相位扰动的全局缩放因子。
        - 当使用随机相位时，它控制扰动范围 [0, `magnitude` * 2 * `pi`]。
        - 当手动指定相位时，它作为乘数应用于给定的相位。默认为 1.0。
    - `random_state` : `int | torch.Generator`, optional
        当`phase`为`None`时，用于生成随机相位的种子。默认为 `None`。
    - `phase` : `float | list | torch.Tensor`, optional
        手动指定的相位（以弧度为单位）。
        - 如果是 `float`，所有频率分量将使用相同的相位。
        - 如果是 `list/tensor`，则为每个正频率分量指定相位。
          其长度必须为 `(n_samples - 1) // 2` (奇数) 或 `n_samples // 2 - 1` (偶数)。
        - 如果为 `None` (默认)，则生成随机相位。

    Returns
    -------
    - `torch.Tensor`
        变换后的输入数据，形状与输入相同。
    """
    device = X.device
    needs_squeeze = X.ndim != 3
    original_shape = X.shape
    X_proc = cast(torch.Tensor, ensure_3dims(X, newaxis='batch'))

    f = torch.fft.fft(X_proc, dim=-1)

    n_samples = X_proc.shape[-1]
    batch_size = X_proc.shape[0]
    base_channels = 1  # 为所有通道生成一个相位模式，确保通道同步性不被破坏

    # 根据信号长度确定需要的相位值的数量
    n_positive_freqs = (n_samples - 1) // 2 if n_samples % 2 != 0 else n_samples // 2 - 1

    # --- 步骤1: 准备正频率部分的相位数据 ---
    if phase is None:
        # 模式1: 生成随机相位
        if not (0 <= phase_noise_magnitude <= 1):
            raise ValueError(
                f'当使用随机相位时, phase_noise_magnitude 必须是 0到1之间的浮点数。得到 {phase_noise_magnitude}。'
            )
        generator = _use_random_state(random_state, device=device)
        random_values = torch.rand(
            (batch_size, base_channels, n_positive_freqs),
            generator=generator,
            device=device,
        )
        positive_phases = phase_noise_magnitude * 2 * torch.pi * random_values * 1j
    else:
        # 模式2: 处理手动指定的相位
        if isinstance(phase, (float, int)):
            phase_values = torch.full((n_positive_freqs,), float(phase), device=device)
        elif isinstance(phase, list):
            phase_values = torch.as_tensor(phase, dtype=torch.float32, device=device)
            if phase_values.size(0) != n_positive_freqs:
                raise ValueError(
                    f"对于长度为 {n_samples} 的信号, 'phase' 数组需要包含 "
                    f'{n_positive_freqs} 个元素, 但收到了 {phase_values.size(0)} 个。'
                )
        elif isinstance(phase, torch.Tensor):
            phase_values = phase.to(device=device)
            if phase_values.size(0) != n_positive_freqs:
                raise ValueError(
                    f"对于长度为 {n_samples} 的信号, 'phase' 张量需要包含 "
                    f'{n_positive_freqs} 个元素, 但收到了 {phase_values.size(0)} 个。'
                )
        else:
            raise TypeError(
                f"'phase' 必须是 float, int, list 或 torch tensor, 但收到 {type(phase)}。"
            )

        # 应用全局缩放并转换为复数形式
        positive_phases = phase_noise_magnitude * phase_values.reshape(1, 1, -1) * 1j

    # --- 步骤2: 调用核心函数构建完整的对称相位 ---
    phase_perturbation = _construct_symmetric_phase(positive_phases, n_samples)

    # 将单通道相位模式复制到所有实际通道
    if X_proc.shape[1] > 1:
        phase_perturbation = phase_perturbation.repeat(1, X_proc.shape[1], 1)

    # 应用相位噪声
    f_shifted = f * torch.exp(phase_perturbation)

    # IFFT
    shifted = torch.fft.ifft(f_shifted, dim=-1)
    transformed_X = shifted.real

    # 恢复原始形状
    if needs_squeeze:
        transformed_X = transformed_X.reshape(original_shape)

    return transformed_X



"""
函数式编程工具包
提供管道组合、函数包装、Monad 等函数式编程抽象
"""

import typing

import abc
from dataclasses import dataclass
from functools import reduce, wraps
from typing import Any, Callable, Generic, TypeVar, ParamSpec

__all__ = [
    'Pipeline',
    'PipelineExecution',
    'pipe',
    'FuncWrapper',
    'Result',
    'Maybe',
    'Validation',
    'returns_result',
    'returns_maybe',
    'safe',
    'nullable',
]

P = ParamSpec('P')
V = TypeVar('V')
E = TypeVar('E')
T = TypeVar('T')
U = TypeVar('U')


class PipelineError(Exception):
    """管道执行错误"""

    def __init__(self, func_name: str, step: int, original_error: Exception):
        self.func_name = func_name
        self.step = step
        self.original_error = original_error
        super().__init__(f"步骤 {step} 中函数 '{func_name}' 执行失败: {original_error}")


@dataclass(frozen=True)
class PipelineExecution(Generic[T]):
    """
    管道执行结果封装（线程安全）

    Attributes
    ----------
    - `result`: 管道最终返回值
    - `trace`: 每一步的中间结果列表
    """

    result: T
    trace: list[Any]


class Pipeline:
    def __init__(self, *funcs: Callable):
        """
        初始化函数管道

        将多个函数串联执行，前一个函数的输出作为后一个函数的输入

        特性：
        - 线程安全：不存储执行状态到实例
        - 提供详细的错误追踪
        - 支持 `|` 操作符语法
        - 使用 `run()` 方法可获取执行轨迹

        Parameters
        ----------
        - `funcs`: `Callable`, 要串联的函数序列

        Examples
        ----------
        ```py
        >>> def double(x):
        ...     return x * 2
        >>> def add_one(x):
        ...     return x + 1
        >>>
        >>> pipeline = Pipeline(double, add_one)
        >>> result = pipeline(3)
        >>> print(result)  # 输出: 7
        >>>
        >>> # 如需调试信息，使用 run() 方法
        >>> execution = pipeline.run(3)
        >>> print(execution.result)  # 7
        >>> print(execution.trace)   # [6, 7]
        ```
        """
        if not funcs:
            raise ValueError('Pipeline 至少需要一个函数')
        if not all(callable(f) for f in funcs):
            raise TypeError('所有参数必须是可调用对象')
        self.funcs = funcs

    def run(self, *args: Any, **kwargs: Any) -> PipelineExecution:
        """
        执行管道，返回包含结果和执行轨迹的 PipelineExecution 对象

        Parameters
        ----------
        - `args`: 传递给第一个函数的位置参数
        - `kwargs`: 传递给第一个函数的关键字参数

        Returns
        ----------
        - `PipelineExecution`, 包含 result 和 trace 的执行结果
        """
        trace: list[Any] = []

        # 第一个函数使用原始参数调用
        try:
            current_value = self.funcs[0](*args, **kwargs)
            trace.append(current_value)
        except Exception as e:
            raise PipelineError(
                func_name=getattr(self.funcs[0], '__name__', repr(self.funcs[0])),
                step=1,
                original_error=e,
            ) from e

        # 后续函数逐个调用，不进行隐式解包
        for i, func in enumerate(self.funcs[1:], start=2):
            try:
                current_value = func(current_value)
                trace.append(current_value)
            except Exception as e:
                raise PipelineError(
                    func_name=getattr(func, '__name__', repr(func)),
                    step=i,
                    original_error=e,
                ) from e

        return PipelineExecution(result=current_value, trace=trace)

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        """执行管道，直接返回最终结果"""
        return self.run(*args, **kwargs).result

    def __or__(self, other: Callable) -> 'Pipeline':
        """支持 `pipeline | func` 语法"""
        if isinstance(other, Pipeline):
            return Pipeline(*self.funcs, *other.funcs)
        return Pipeline(*self.funcs, other)

    def __repr__(self) -> str:
        func_names = [getattr(f, '__name__', repr(f)) for f in self.funcs]
        return f"Pipeline({' -> '.join(func_names)})"


class FuncWrapper:
    def __init__(
        self,
        func: Callable,
        fn_args: tuple | None = None,
        call_alias: str | typing.Sequence[str] | None = None,
        **fn_kwargs,
    ) -> None:
        """
        初始化函数包装器

        预设参数并支持别名调用

        Parameters
        ----------
        - `func`: `Callable`, 要包装的函数
        - `fn_args`: `tuple | None`, 预设的位置参数（会追加到调用参数后）, by default None
        - `call_alias`: `str | list[str] | None`, 可通过别名调用，例如 `wrapper.run()` 等价于 `wrapper()`, by default None
        - `fn_kwargs`: 预设的关键字参数（调用时的参数优先级更高）

        Examples
        ----------
        ```py
        >>> def greet(name, greeting="Hello"):
        ...     return f"{greeting}, {name}!"
        >>>
        >>> greet_casual = FuncWrapper(greet, greeting="Hi", call_alias="run")
        >>> greet_casual("Alice")  # 输出: "Hi, Alice!"
        >>> greet_casual.run("Bob")  # 输出: "Hi, Bob!"
        ```
        """
        self.func = func
        self.preset_args = fn_args or ()
        self.preset_kwargs = fn_kwargs
        self.alias = self._normalize_alias(call_alias)

        # 保留原函数的元数据
        wraps(func)(self)

    @staticmethod
    def _normalize_alias(alias: str | typing.Sequence[str] | None) -> list[str]:
        """标准化别名为列表"""
        if alias is None:
            return []
        if isinstance(alias, str):
            return [alias]
        return list(alias)

    def __getattr__(self, item: str) -> Any:
        """支持别名调用和透传原函数属性"""
        if item in self.alias:
            return self.__call__
        # 尝试从原函数获取属性
        try:
            return getattr(self.func, item)
        except AttributeError:
            raise AttributeError(f"'{type(self).__name__}' 和被包装函数均无属性 '{item}'")

    def __call__(self, *args, **kwargs) -> Any:
        """调用包装函数，合并预设参数"""
        # 预设参数在后，调用参数在前（调用时的参数优先）
        merged_args = args + self.preset_args
        # 调用时的 kwargs 会覆盖预设的 kwargs
        merged_kwargs = {**self.preset_kwargs, **kwargs}
        return self.func(*merged_args, **merged_kwargs)

    def __repr__(self) -> str:
        func_name = getattr(self.func, '__name__', repr(self.func))
        alias_str = f', aliases={self.alias}' if self.alias else ''
        return f'FuncWrapper({func_name}{alias_str})'


def pipe(inputs: Any, *funcs: Callable) -> Any:
    """
    管道函数：将输入依次传递给多个函数

    将输入依次传递给多个函数

    Parameters
    ----------
    - `inputs`: `Any`, 初始输入值
    - `funcs`: `Callable`, 要依次执行的函数序列

    Returns
    ----------
    - `Any`, 最后一个函数的返回值

    Examples
    ----------
    ```py
    >>> pipe(3, lambda x: x * 2, lambda x: x + 1)
    7
    ```
    """
    return reduce(lambda val, func: func(val), funcs, inputs)


# ============================================================================
# Monad 抽象基类
# ============================================================================


class Monad(abc.ABC, Generic[T]):
    """Monad 抽象基类"""

    @abc.abstractmethod
    def bind(self, func: Callable[[T], 'Monad[T]']) -> 'Monad[T]':
        """将函数应用于 Monad 内的值"""
        pass

    @abc.abstractmethod
    def unwrap(self) -> T:
        """提取 Monad 内的值（可能失败）"""
        pass

    def map(self, func: Callable[[T], Any]) -> 'Monad':
        """映射内部值（自动包装返回值）"""
        return self.bind(lambda x: type(self).pure(func(x)))

    @classmethod
    @abc.abstractmethod
    def pure(cls, value: T) -> 'Monad[T]':
        """将值包装进 Monad"""
        pass


# ============================================================================
# Result Monad
# ============================================================================


class Result(Monad[V], Generic[V, E]):
    def __init__(self, value: V | None = None, err: E | None = None):
        """
        初始化 Result Monad

        表示可能成功或失败的计算

        特性：
        - 成功时包含值 (V)
        - 失败时包含错误 (E)
        - 链式调用时自动传播错误

        Parameters
        ----------
        - `value`: `V | None`, 成功时的值, by default None
        - `err`: `E | None`, 失败时的错误, by default None

        Examples
        ----------
        ```py
        >>> def safe_divide(x, y):
        ...     if y == 0:
        ...         return Result.from_err("除数不能为零")
        ...     return Result.from_ok(x / y)
        >>>
        >>> result = Result.from_ok(10).bind(lambda x: safe_divide(x, 2))
        >>> result.unwrap()  # 输出: 5.0
        ```
        """
        if value is None and err is None:
            raise ValueError('Result 必须包含 value 或 err 之一')
        self.value = value
        self.err = err

    @property
    def is_ok(self) -> bool:
        """是否为成功状态"""
        return self.err is None

    @classmethod
    def from_ok(cls, value: V) -> 'Result[V, None]':
        """创建成功的 Result"""
        return typing.cast(Result[V, None], cls(value=value, err=None))

    @classmethod
    def from_err(cls, err: E) -> 'Result[None, E]':
        """创建失败的 Result"""
        return typing.cast(Result[None, E], cls(value=None, err=err))

    @classmethod
    def pure(cls, value: V) -> 'Result[V, None]':
        """将值包装为成功的 Result"""
        return cls.from_ok(value)

    def bind(self, func: Callable[[V], 'Result[Any, E]']) -> 'Result[Any, E]':
        """应用函数（仅在成功时）"""
        if self.is_ok:
            assert self.value is not None
            try:
                return func(self.value)
            except Exception as e:
                return typing.cast(Result[Any, E], Result.from_err(e))
        return typing.cast(Result[Any, E], self)

    def map_err(self, func: Callable[[E], Any]) -> 'Result[V, Any]':
        """映射错误值"""
        if not self.is_ok:
            assert self.err is not None
            return typing.cast(Result[V, Any], Result.from_err(func(self.err)))
        return typing.cast(Result[V, Any], self)

    def unwrap(self) -> V:
        """提取值（失败时抛出异常）"""
        if self.is_ok:
            assert self.value is not None
            return self.value
        raise ValueError(f'无法 unwrap 失败的 Result: {self.err}')

    def unwrap_or(self, default: V) -> V:
        """提取值或返回默认值"""
        if self.is_ok:
            assert self.value is not None
            return self.value
        return default

    def unwrap_or_else(self, func: Callable[[E], V]) -> V:
        """提取值或通过函数计算默认值"""
        if self.is_ok:
            assert self.value is not None
            return self.value
        assert self.err is not None
        return func(self.err)

    def __repr__(self) -> str:
        if self.is_ok:
            return f'Result.Ok({self.value!r})'
        return f'Result.Err({self.err!r})'


# ============================================================================
# Maybe Monad
# ============================================================================


class Maybe(Monad[V], Generic[V]):
    def __init__(self, value: V | None = None):
        """
        初始化 Maybe Monad

        表示可能存在或不存在的值

        特性：
        - Some(value): 包含值
        - None: 不包含值
        - 链式调用时自动处理空值

        Parameters
        ----------
        - `value`: `V | None`, 包含的值, by default None

        Examples
        ----------
        ```py
        >>> def safe_sqrt(x):
        ...     if x < 0:
        ...         return Maybe.none()
        ...     return Maybe.some(x ** 0.5)
        >>>
        >>> Maybe.some(16).bind(safe_sqrt).unwrap()  # 输出: 4.0
        >>> Maybe.some(-4).bind(safe_sqrt).unwrap()  # 抛出异常
        ```
        """
        self._value = value

    @property
    def is_some(self) -> bool:
        """是否包含值"""
        return self._value is not None

    @classmethod
    def some(cls, value: V) -> 'Maybe[V]':
        """创建包含值的 Maybe"""
        if value is None:
            raise ValueError('Maybe.some() 不能接受 None，请使用 Maybe.none()')
        return cls(value)

    @classmethod
    def none(cls) -> 'Maybe[None]':
        """创建空的 Maybe"""
        return typing.cast(Maybe[None], cls(None))

    @classmethod
    def pure(cls, value: V) -> 'Maybe[V]':
        """将值包装为 Maybe（自动处理 None）"""
        if value is not None:
            return cls.some(value)
        return typing.cast(Maybe[V], cls.none())

    def bind(self, func: Callable[[V], 'Maybe[Any]']) -> 'Maybe[Any]':
        """应用函数（仅在包含值时）"""
        if self.is_some:
            assert self._value is not None
            try:
                return func(self._value)
            except Exception:
                return Maybe.none()
        return typing.cast(Maybe[Any], self)

    def unwrap(self) -> V:
        """提取值（空值时抛出异常）"""
        if self.is_some:
            assert self._value is not None
            return self._value
        raise ValueError('无法 unwrap 空的 Maybe')

    def unwrap_or(self, default: V) -> V:
        """提取值或返回默认值"""
        if self.is_some:
            assert self._value is not None
            return self._value
        return default

    def unwrap_or_else(self, func: Callable[[], V]) -> V:
        """提取值或通过函数计算默认值"""
        if self.is_some:
            assert self._value is not None
            return self._value
        return func()

    def __repr__(self) -> str:
        if self.is_some:
            return f'Maybe.Some({self._value!r})'
        return 'Maybe.None'


# ============================================================================
# Validation Monad
# ============================================================================


class Validation(Monad[V], Generic[V, E]):
    def __init__(self, value: V | None = None, errors: typing.Sequence[E] | None = None):
        """
        初始化 Validation Monad

        支持累积多个错误（通过 `combine` 方法）

        特性：
        - Success：包含值
        - Failure：包含错误列表
        - `bind` 依然是短路的（因为后续步骤依赖前一步的输出）
        - `combine` 方法支持并行验证并累积所有错误
        - 适用于表单验证等需要显示所有错误的场景

        Parameters
        ----------
        - `value`: `V | None`, 成功时的值, by default None
        - `errors`: `list[E] | None`, 失败时的错误列表, by default None

        Examples
        ----------
        ```py
        >>> def validate_age(age):
        ...     if age < 0:
        ...         return Validation.failure(["年龄不能为负数"])
        ...     if age > 150:
        ...         return Validation.failure(["年龄不合理"])
        ...     return Validation.success(age)
        >>>
        >>> # 可以组合多个验证并累积错误
        >>> result = validate_age(-5)
        >>> result.errors  # ["年龄不能为负数"]
        ```
        """
        if value is None and not errors:
            raise ValueError('Validation 必须包含 value 或 errors')
        self.value = value
        self.errors = errors or []

    @property
    def is_success(self) -> bool:
        """是否验证成功"""
        return not self.errors

    @classmethod
    def success(cls, value: V) -> 'Validation[V, E]':
        """创建成功的 Validation"""
        return cls(value=value, errors=[])

    @classmethod
    def failure(cls, errors: typing.Sequence[E]) -> 'Validation[None, E]':
        """创建失败的 Validation"""
        return typing.cast(Validation[None, E], cls(value=None, errors=errors))

    @classmethod
    def pure(cls, value: V) -> 'Validation[V, E]':
        """将值包装为成功的 Validation"""
        return cls.success(value)

    def bind(self, func: Callable[[V], 'Validation[U, E]']) -> 'Validation[U, E]':
        """应用函数（累积错误）"""
        if self.is_success:
            assert self.value is not None
            try:
                result = func(self.value)
                # 即使当前成功，也要保留之前累积的错误
                if not result.is_success:
                    return typing.cast(
                        Validation[U, E],
                        Validation.failure(list(self.errors) + list(result.errors)),
                    )
                return result
            except Exception as e:
                return typing.cast(
                    Validation[U, E],
                    Validation.failure(list(self.errors) + [typing.cast(E, e)]),
                )
        return typing.cast(Validation[U, E], self)

    def combine(self, other: 'Validation[U, E]') -> 'Validation[tuple[V, U], E]':
        """组合两个 Validation（累积双方错误）"""
        if self.is_success and other.is_success:
            assert self.value is not None
            assert other.value is not None
            return Validation.success((self.value, other.value))
        return typing.cast(
            Validation[tuple[V, U], E],
            Validation.failure(list(self.errors) + list(other.errors)),
        )

    def unwrap(self) -> V:
        """提取值（失败时抛出异常）"""
        if self.is_success:
            assert self.value is not None
            return self.value
        raise ValueError(f'无法 unwrap 失败的 Validation，错误: {self.errors}')

    def unwrap_or(self, default: V) -> V:
        """提取值或返回默认值"""
        if self.is_success:
            assert self.value is not None
            return self.value
        return default

    def __repr__(self) -> str:
        if self.is_success:
            return f'Validation.Success({self.value!r})'
        return f'Validation.Failure({self.errors!r})'


# ============================================================================
# 装饰器工具
# ============================================================================


def returns_result(func: Callable[P, V]) -> Callable[P, Result[V, Exception]]:
    """
    装饰器：将函数返回值包装为 Result

    捕获异常并转换为 Result.Err

    Parameters
    ----------
    - `func`: `Callable[P, V]`, 要包装的函数

    Returns
    ----------
    - `Callable[P, Result[V, Exception]]`, 返回 Result 的函数

    Examples
    ----------
    ```py
    >>> @returns_result
    ... def divide(x, y):
    ...     return x / y
    >>>
    >>> result = divide(10, 2)
    >>> result.unwrap()  # 5.0
    >>> result = divide(10, 0)
    >>> result.is_ok  # False
    ```
    """

    @wraps(func)
    def wrapper(*args: P.args, **kwargs: P.kwargs) -> Result[V, Exception]:
        try:
            return typing.cast(
                Result[V, Exception], Result.from_ok(func(*args, **kwargs))
            )
        except Exception as e:
            return typing.cast(Result[V, Exception], Result.from_err(e))

    return wrapper


def returns_maybe(func: Callable[P, V | None]) -> Callable[P, Maybe[V]]:
    """
    装饰器：将函数返回值包装为 Maybe

    自动将 None 转换为 Maybe.none()

    Parameters
    ----------
    - `func`: `Callable[P, V | None]`, 可能返回 None 的函数

    Returns
    ----------
    - `Callable[P, Maybe[V]]`, 返回 Maybe 的函数

    Examples
    ----------
    ```py
    >>> @returns_maybe
    ... def find_user(user_id):
    ...     users = {1: "Alice", 2: "Bob"}
    ...     return users.get(user_id)
    >>>
    >>> result = find_user(1)
    >>> result.unwrap()  # "Alice"
    >>> result = find_user(999)
    >>> result.is_some  # False
    ```
    """

    @wraps(func)
    def wrapper(*args: P.args, **kwargs: P.kwargs) -> Maybe[V]:
        result = func(*args, **kwargs)
        return Maybe.pure(result)

    return wrapper


def safe(func: Callable[P, V]) -> Callable[P, Result[V, Exception]]:
    """
    装饰器：安全执行函数（等同于 returns_result）

    提供语义更清晰的别名

    Examples
    ----------
    ```py
    >>> @safe
    ... def parse_int(s):
    ...     return int(s)
    >>>
    >>> parse_int("123").unwrap()  # 123
    >>> parse_int("abc").is_ok  # False
    ```
    """
    return returns_result(func)


def nullable(func: Callable[P, V | None]) -> Callable[P, Maybe[V]]:
    """
    装饰器：处理可空返回值（等同于 returns_maybe）

    提供语义更清晰的别名

    Examples
    ----------
    ```py
    >>> @nullable
    ... def get_config(key):
    ...     config = {"debug": True}
    ...     return config.get(key)
    >>>
    >>> get_config("debug").unwrap()  # True
    >>> get_config("missing").unwrap_or(False)  # False
    ```
    """
    return returns_maybe(func)



import typing
import torch
from torch import nn

from . import base


class DepthwiseSeperableConv(base.BaseModule):
    """深度可分离卷积层

    将标准卷积分解为深度卷积（Depthwise）和逐点卷积（Pointwise）两个步骤，
    大幅减少参数量和计算量，同时保持相似的表达能力。

    深度卷积：每个输入通道独立进行卷积
    逐点卷积：使用 1x1 卷积进行通道间的线性组合

    Examples
    ----------
    ```py
    # 1D EEG 信号处理
    ds_conv = DepthwiseSeperableConv(
        in_channels=64, out_channels=128, kernel_size=3,
        stride=1, padding=1, Conv=nn.Conv1d
    )
    eeg_data = torch.randn(32, 64, 1000)  # [N, C, T]
    output = ds_conv(eeg_data)  # [N, 128, 1000]

    # 2D 图像处理
    ds_conv_2d = DepthwiseSeperableConv(
        in_channels=256, out_channels=512, kernel_size=3,
        stride=2, padding=1, Conv=nn.Conv2d
    )
    image = torch.randn(16, 256, 32, 32)  # [N, C, H, W]
    features = ds_conv_2d(image)  # [N, 512, 16, 16]
    ```
    """

    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int,
        stride: int = 1,
        padding: int = 0,
        dilation: int = 1,
        bias: bool = False,
        Conv=nn.Conv1d,
    ):
        """初始化 DepthwiseSeperableConv 层

        Parameters
        ----------
        - `in_channels` : `int`
            - 输入通道数
        - `out_channels` : `int`
            - 输出通道数
        - `kernel_size` : `int`
            - 卷积核大小
        - `stride` : `int`, optional
            - 步长, by default 1
        - `padding` : `int`, optional
            - 填充, by default 0
        - `dilation` : `int`, optional
            - 膨胀率, by default 1
        - `bias` : `bool`, optional
            - 是否使用偏置, by default False
        - `Conv` : `nn.Module`, optional
            - 卷积层类型 (nn.Conv1d, nn.Conv2d, nn.Conv3d), by default nn.Conv1d
        """
        super(DepthwiseSeperableConv, self).__init__()
        self.depthwise_conv = Conv(
            in_channels,
            in_channels,
            kernel_size,
            stride,
            padding,
            dilation,
            in_channels,
            bias,
        )
        self.pointwise_conv = Conv(in_channels, out_channels, 1, 1, 0, 1, 1, bias)

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        """前向传播

        Parameters
        ----------
        - `x` : `torch.Tensor`
            - 输入张量，形状取决于卷积类型：
              - 1D: [N, C, T]
              - 2D: [N, C, H, W]
              - 3D: [N, C, D, H, W]

        Returns
        ----------
        - `torch.Tensor`
            - 输出张量，形状为 [N, out_channels, ...]
        """
        x = self.depthwise_conv(x)
        x = self.pointwise_conv(x)
        return x


class MaxNormConv(base.BaseModule):
    """最大范数约束卷积层

    在每次前向传播时对卷积层的权重进行最大范数约束，
    防止权重过大导致的梯度爆炸问题，提高训练稳定性。

    特别适用于 EEG 信号分析中的空间滤波器学习，
    可以防止某些电极权重过大而导致的过拟合。

    Examples
    ----------
    ```py
    # EEG 空间滤波器
    spatial_conv = nn.Conv1d(64, 32, kernel_size=1)
    max_norm_conv = MaxNormConv(conv=spatial_conv, max_norm=1.0)
    eeg_data = torch.randn(32, 64, 1000)  # [N, C, T]
    filtered_eeg = max_norm_conv(eeg_data)  # [N, 32, 1000]

    # 2D 卷积的最大范数约束
    conv2d = nn.Conv2d(128, 256, kernel_size=3, padding=1)
    max_norm_conv2d = MaxNormConv(conv=conv2d, max_norm=2.0)
    features = torch.randn(16, 128, 32, 32)
    output = max_norm_conv2d(features)
    ```
    """

    def __init__(self, conv, max_norm: float = 1.0):
        """初始化 MaxNormConv 层

        Parameters
        ----------
        - `conv` : `base.Conv`
            - 要应用最大范数约束的卷积层
        - `max_norm` : `float`, optional
            - 最大范数值, by default 1.0
        """
        super(MaxNormConv, self).__init__()
        self.conv = conv
        self.max_norm = max_norm

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        """前向传播

        在每次前向传播前对卷积权重进行最大范数约束。

        Parameters
        ----------
        - `x` : `torch.Tensor`
            - 输入张量，形状取决于卷积类型：
              - 1D: [N, C, T]
              - 2D: [N, C, H, W]
              - 3D: [N, C, D, H, W]

        Returns
        ----------
        - `torch.Tensor`
            - 卷积输出，权重已应用最大范数约束
        """
        # 应用最大范数约束到权重
        self.conv.weight.data = torch.renorm(
            self.conv.weight.data, p=2, dim=0, maxnorm=self.max_norm
        )
        return self.conv.forward(x)


class SincConv1d(base.BaseModule):
    """可学习的带通滤波器卷积层，专用于信号处理

    基于 Sinc 函数的卷积层，能够学习特定频段的滤波器参数，
    特别适用于语音和脑电信号处理。相比传统卷积，SincConv 能够
    学习更有意义的频域特征，提高信号分类和分析的性能。

    References
    ----------
    - **Paper**: Ravanelli, M., & Bengio, Y. (2018). "Speaker Recognition from Raw Waveform with SincNet."
      *Proceedings of SLT 2018*. https://arxiv.org/abs/1808.00158
    - **Official Implementation**: https://github.com/mravanelli/SincNet
    - **PyTorch Implementation**: https://github.com/pyannote/pyannote-audio
    - **EEG Application**: Borra, D., et al. (2021). "A Lightweight Multi-Scale Convolutional Neural Network for P300 Detection."

    Examples
    ----------
    ```py
    # EEG 信号滤波 (250Hz 采样率)
    sinc_conv = SincConv1d(in_channels=64, out_channels=32, kernel_size=251, sample_rate=250)
    eeg_data = torch.randn(32, 64, 1000)  # [N, C, T]
    filtered = sinc_conv(eeg_data)

    # 检查学习到的频率范围
    print(f"Low cutoff frequencies: {sinc_conv.low_hz}")
    print(f"High cutoff frequencies: {sinc_conv.high_hz}")

    # 语音信号处理 (16kHz 采样率)
    sinc_conv_audio = SincConv1d(in_channels=1, out_channels=80, kernel_size=251, sample_rate=16000)
    audio_data = torch.randn(16, 1, 16000)  # 1秒音频
    features = sinc_conv_audio(audio_data)
    ```
    """

    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int,
        sample_rate: int = 250,
        min_low_hz: float = 50,
        min_band_hz: float = 50,
        stride: int = 1,
        padding: int = 0,
        dilation: int = 1,
        bias: bool = False,
    ):
        """初始化 SincConv1d 层

        Parameters
        ----------
        - `in_channels` : `int`
            - 输入通道数
        - `out_channels` : `int`
            - 输出通道数（滤波器数量）
        - `kernel_size` : `int`
            - 卷积核大小，必须为奇数
        - `sample_rate` : `int`, optional
            - 采样率，用于频率计算, by default 250
        - `min_low_hz` : `float`, optional
            - 最小低截止频率, by default 50
        - `min_band_hz` : `float`, optional
            - 最小带宽, by default 50
        - `stride` : `int`, optional
            - 步长, by default 1
        - `padding` : `int`, optional
            - 填充, by default 0
        - `dilation` : `int`, optional
            - 膨胀, by default 1
        - `bias` : `bool`, optional
            - 是否使用偏置, by default False
        """
        super().__init__()

        if kernel_size % 2 == 0:
            raise ValueError('kernel_size must be odd')

        self.in_channels = in_channels
        self.out_channels = out_channels
        self.kernel_size = kernel_size
        self.sample_rate = sample_rate
        self.min_low_hz = min_low_hz
        self.min_band_hz = min_band_hz
        self.stride = stride
        self.padding = padding
        self.dilation = dilation

        # 频率参数（可学习）
        self.low_hz_ = nn.Parameter(
            torch.Tensor(out_channels).uniform_(min_low_hz, sample_rate / 2 - min_band_hz)
        )
        self.band_hz_ = nn.Parameter(
            torch.Tensor(out_channels).uniform_(min_band_hz, sample_rate / 2 - min_low_hz)
        )

        # 窗函数
        n_lin = torch.linspace(0, (kernel_size / 2) - 1, steps=int((kernel_size / 2)))
        self.window_ = 0.54 - 0.46 * torch.cos(2 * torch.pi * n_lin / kernel_size)

        # 时间轴
        n = (kernel_size - 1) / 2.0
        self.n_ = 2 * torch.pi * torch.arange(-n, 0).view(1, -1) / sample_rate

        if bias:
            self.bias = nn.Parameter(torch.Tensor(out_channels))
        else:
            self.register_parameter('bias', None)

        self.init_weights()

    def init_weights(self):
        """初始化权重"""
        if self.bias is not None:
            nn.init.constant_(self.bias, 0.0)

    @property
    def low_hz(self) -> torch.Tensor:
        """获取低截止频率"""
        return torch.abs(self.low_hz_) + self.min_low_hz

    @property
    def high_hz(self) -> torch.Tensor:
        """获取高截止频率"""
        return torch.clamp(
            self.low_hz + torch.abs(self.band_hz_) + self.min_band_hz,
            self.min_low_hz,
            self.sample_rate / 2,
        )

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        """前向传播

        Parameters
        ----------
        - `x` : `torch.Tensor`
            - 输入信号，形状为 [N, C, T]

        Returns
        ----------
        - `torch.Tensor`
            - 滤波后的信号
        """
        # 移动设备
        n_ = self.n_.to(x.device)
        window_ = self.window_.to(x.device)

        low = self.low_hz.view(-1, 1)
        high = self.high_hz.view(-1, 1)

        # 计算 sinc 滤波器
        f_times_t_low = torch.matmul(low, n_)
        f_times_t_high = torch.matmul(high, n_)

        band_pass_left = (
            (torch.sin(f_times_t_high) - torch.sin(f_times_t_low)) / (n_ / 2)
        ) * window_
        band_pass_center = 2 * (high - low)
        band_pass_right = torch.flip(band_pass_left, dims=[1])

        band_pass = torch.cat([band_pass_left, band_pass_center, band_pass_right], dim=1)
        band_pass = band_pass / (2 * high)

        # 重塑滤波器以适应标准卷积
        # 每个输出通道对应一个独立的带通滤波器
        filters = band_pass.view(self.out_channels, 1, self.kernel_size)

        # 扩展到所有输入通道 - 每个输出滤波器应用到所有输入通道
        filters = filters.repeat(1, self.in_channels, 1)

        return nn.functional.conv1d(
            x,
            filters,
            bias=self.bias,
            stride=self.stride,
            padding=self.padding,
            dilation=self.dilation,
            groups=1,
        )

    def extra_repr(self) -> str:
        return (
            f'in_channels={self.in_channels}, out_channels={self.out_channels}, '
            f'kernel_size={self.kernel_size}, sample_rate={self.sample_rate}'
        )


class MultiScaleConv1d(base.BaseModule):
    """多尺度时序卷积模块

    通过并行的不同扩张率卷积捕获多个时间尺度的特征，
    特别适用于时序信号分析，如脑电信号的多频段特征提取。

    References
    ----------
    - **Paper**: Oord, A. v. d., et al. (2016). "WaveNet: A Generative Model for Raw Audio."
      *ICML 2016*. https://arxiv.org/abs/1609.03499
    - **Temporal Convolutional Networks**: Bai, S., et al. (2018). "An Empirical Evaluation of Generic Convolutional and Recurrent Networks for Sequence Modeling."
      https://arxiv.org/abs/1803.01271
    - **EEG Application**: Lawhern, V. J., et al. (2018). "EEGNet: a compact convolutional neural network for EEG-based brain–computer interfaces."

    Examples
    ----------
    ```py
    # EEG 多尺度特征提取
    multi_conv = MultiScaleConv1d(in_channels=64, out_channels=128, kernel_size=3, dilations=[1, 2, 4, 8])
    eeg_data = torch.randn(32, 64, 1000)  # [N, C, T]
    multi_features = multi_conv(eeg_data)

    # 自定义扩张率
    custom_conv = MultiScaleConv1d(in_channels=32, out_channels=64, kernel_size=5, dilations=[1, 3, 9, 27])
    features = custom_conv(torch.randn(16, 32, 500))
    ```
    """

    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int,
        dilations: typing.Sequence = [1, 2, 4, 8],
        activation: str = 'relu',
        normalization: bool = True,
    ):
        """初始化 MultiScaleConv1d 层

        Parameters
        ----------
        - `in_channels` : `int`
            - 输入通道数
        - `out_channels` : `int`
            - 输出通道数
        - `kernel_size` : `int`
            - 卷积核大小
        - `dilations` : `list`, optional
            - 扩张率列表, by default [1, 2, 4, 8]
        - `activation` : `str`, optional
            - 激活函数类型, by default 'relu'
        - `normalization` : `bool`, optional
            - 是否使用批归一化, by default True
        """
        super().__init__()

        self.dilations = dilations
        self.num_scales = len(dilations)

        # 每个尺度的输出通道数
        channels_per_scale = out_channels // self.num_scales
        remaining_channels = out_channels % self.num_scales

        self.convs = nn.ModuleList()
        self.norms = nn.ModuleList() if normalization else None

        for i, dilation in enumerate(dilations):
            # 分配通道数
            scale_out_channels = channels_per_scale + (1 if i < remaining_channels else 0)

            # 计算padding以保持时序长度
            padding = (kernel_size - 1) * dilation // 2

            conv = nn.Conv1d(
                in_channels,
                scale_out_channels,
                kernel_size,
                padding=padding,
                dilation=dilation,
                bias=not normalization,
            )
            self.convs.append(conv)

            if normalization:
                assert self.norms is not None
                self.norms.append(nn.BatchNorm1d(scale_out_channels))

        # 激活函数
        if activation == 'relu':
            self.activation = nn.ReLU(inplace=True)
        elif activation == 'gelu':
            self.activation = nn.GELU()
        elif activation == 'swish':
            self.activation = nn.SiLU()
        else:
            self.activation = nn.Identity()

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        """前向传播

        Parameters
        ----------
        - `x` : `torch.Tensor`
            - 输入信号，形状为 [N, C, T]

        Returns
        ----------
        - `torch.Tensor`
            - 多尺度特征，形状为 [N, out_channels, T]
        """
        outputs = []

        for i, conv in enumerate(self.convs):
            out = conv(x)

            if self.norms is not None:
                out = self.norms[i](out)

            out = self.activation(out)
            outputs.append(out)

        # 拼接所有尺度的特征
        return torch.cat(outputs, dim=1)

    def extra_repr(self) -> str:
        return f'dilations={self.dilations}, num_scales={self.num_scales}'



"""
贝叶斯神经网络系列算法
实现 MC Dropout 和 Deep Ensemble 两种不确定性估计方法

理论基础:
- MC Dropout: 通过测试时启用 Dropout 进行 Monte Carlo 采样估计不确定性
- Deep Ensemble: 通过训练多个独立模型的集成来估计不确定性

参考:
- Gal, Y., & Ghahramani, Z. (2016). Dropout as a bayesian approximation: Representing model uncertainty in deep learning.
- Lakshminarayanan, B., Pritzel, A., & Blundell, C. (2017). Simple and scalable predictive uncertainty estimation using deep ensembles.
"""

import typing

from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    List,
    Literal,
    Optional,
    Tuple,
    Union,
)

import torch
import torch.nn as nn
import torch.nn.functional as F
from torch import Tensor

from zyplib.nn.modules import BaseModule, get_activation
from zyplib.utils.print import warning

if TYPE_CHECKING:
    from zyplib.train.common import FabricTrainingContext


@dataclass(frozen=True)
class BNNOutput:
    """贝叶斯神经网络的不确定性输出

    理论精确的设计：不强行区分不确定性类型，专注于实际可测量的量
    """

    predictive_mean: Tensor  # 预测均值 E[y|x,D]
    predictive_variance: Tensor  # 预测方差 Var[y|x,D]
    prediction_samples: Tensor  # 预测样本 [num_samples, batch_size, ...]


class MCDropoutLayer(nn.Module):
    """Monte Carlo Dropout 层

    可复用的 Dropout 层，在训练和推理时都保持激活状态
    """

    def __init__(self, p: float = 0.5, inplace: bool = False):
        super().__init__()
        if p < 0 or p > 1:
            raise ValueError(f'Dropout 概率必须在 [0, 1] 范围内，得到 {p}')
        self.p = p
        self.inplace = inplace

    def forward(self, x: Tensor) -> Tensor:
        """前向传播，始终应用 Dropout"""
        # 关键：无论 self.training 状态如何，都强制应用 dropout
        return F.dropout(x, p=self.p, training=True, inplace=self.inplace)

    def extra_repr(self) -> str:
        return f'p={self.p}, inplace={self.inplace}'


class MCDropoutNN(BaseModule):
    """Monte Carlo Dropout 神经网络

    通过在推理时保持 Dropout 激活并进行多次前向传播来估计不确定性。
    理论上等价于对深度高斯过程的变分推断。

    Parameters
    ----------
    - `backbone` : `nn.Module`
        - 主干网络
    - `dropout_rate` : `float`, optional
        - Dropout 概率, by default 0.1
    - `auto_mcdp` : `Literal[None, 'insert', 'convert']`, optional
        - MC Dropout 处理模式, by default None
        - None: 不进行任何自动处理，假设模型已包含 MCDropoutLayer
        - 'insert': 在线性层和卷积层后自动插入 MCDropoutLayer
        - 'convert': 将现有的 Dropout 层转换为 MCDropoutLayer
    - `input_shape` : `Tuple[int, ...]`, optional
        - 输入形状，用于模型验证, by default None
    - `validate_model` : `bool`, optional
        - 是否验证模型, by default True

    Examples
    ----------
    ```python
    # 模式1: 使用已包含 MCDropoutLayer 的模型
    backbone = nn.Sequential(
        nn.Linear(784, 128),
        nn.ReLU(),
        MCDropoutLayer(0.1),
        nn.Linear(128, 10)
    )
    mc_model = MCDropoutNN(backbone, auto_mcdp=None)

    # 模式2: 自动插入 MCDropoutLayer
    simple_backbone = nn.Sequential(
        nn.Linear(784, 128),
        nn.ReLU(),
        nn.Linear(128, 10)
    )
    mc_model = MCDropoutNN(simple_backbone, dropout_rate=0.2, auto_mcdp='insert')

    # 模式3: 转换现有 Dropout 为 MCDropoutLayer
    dropout_backbone = nn.Sequential(
        nn.Linear(784, 128),
        nn.Dropout(0.1),
        nn.ReLU(),
        nn.Linear(128, 10),
        nn.Dropout(0.2)
    )
    mc_model = MCDropoutNN(dropout_backbone, auto_mcdp='convert')

    # 使用
    uncertainty_output = mc_model.predict_bnn(x, num_samples=50)
    ```
    """

    BNN_TYPE = 'MCDropoutNN'

    def __init__(
        self,
        backbone: nn.Module,
        dropout_rate: float = 0.1,
        auto_mcdp: Optional[Literal['insert', 'convert']] = None,
        activation: Optional[
            Literal[
                'relu', 'leaky_relu', 'prelu', 'gelu', 'silu', 'tanh', 'sigmoid', 'swish'
            ]
        ]
        | None = None,
    ):
        # 处理 MC Dropout 自动化
        if auto_mcdp == 'insert':
            backbone = self._insert_mc_dropout_layers(backbone, dropout_rate)
        elif auto_mcdp == 'convert':
            converted_count = self._convert_dropout_layers(backbone)
            if converted_count == 0:
                warning('模型中未找到任何 Dropout 层进行转换')

        super().__init__()
        self.backbone = backbone
        self.dropout_rate = dropout_rate
        self.auto_mcdp = auto_mcdp
        self.activation = get_activation(activation, return_type='function') or (
            lambda x: x
        )

    def _insert_mc_dropout_layers(
        self, model: nn.Module, dropout_rate: float
    ) -> nn.Module:
        """在线性层后自动插入 MCDropoutLayer"""
        layers = []

        for name, module in model.named_children():
            layers.append(module)

            # 在线性层和卷积层后插入 Dropout
            if isinstance(module, (nn.Linear, nn.Conv1d, nn.Conv2d, nn.Conv3d)):
                layers.append(MCDropoutLayer(p=dropout_rate))

        return nn.Sequential(*layers)

    def _convert_dropout_layers(self, model: nn.Module) -> int:
        """将现有 Dropout 层转换为 MCDropoutLayer"""
        return MCDropoutNN.apply_mcdropout(model)

    @staticmethod
    def apply_mcdropout(model: nn.Module) -> int:
        """静态方法：将模型中的 Dropout 层转换为 MCDropoutLayer

        借鉴案例代码的 apply_mcdropout 实现，提供独立的转换功能

        Parameters
        ----------
        - `model` : `nn.Module`
            - 要转换的模型

        Returns
        ----------
        - `int`
            - 转换的 Dropout 层数量

        Examples
        ----------
        ```python
        # 直接转换现有模型
        model = EEGNet(channels=8, samples=1000, n_class=2)
        converted_count = MCDropoutNN.apply_mcdropout(model)
        print(f"转换了 {converted_count} 个 Dropout 层")

        # 然后可以直接使用转换后的模型
        mc_model = MCDropoutNN(model, auto_mcdp=None)
        ```
        """
        converted_count = 0

        for name, module in model.named_modules():
            if isinstance(module, (nn.Dropout, nn.Dropout1d, nn.Dropout2d, nn.Dropout3d)):
                parent_name = '.'.join(name.split('.')[:-1])
                layer_name = name.split('.')[-1]

                try:
                    # 递归查找父模块
                    parent = model
                    if parent_name:
                        for part in parent_name.split('.'):
                            parent = getattr(parent, part)

                    # 创建 MCDropoutLayer 替换原 Dropout
                    mc_dropout = MCDropoutLayer(
                        p=module.p, inplace=getattr(module, 'inplace', False)
                    )
                    setattr(parent, layer_name, mc_dropout)
                    converted_count += 1

                except AttributeError as e:
                    warning(f"无法转换 Dropout 层 '{name}': {e}")
                    continue

        return converted_count

    def forward(self, x: Tensor) -> Tensor:
        """标准前向传播"""
        logits = self.backbone(x)
        return self.activation(logits)

    def predict_bnn(
        self, x: Tensor, num_samples: int = 100, *, CHUNCK_SIZE=20
    ) -> BNNOutput:
        # self.eval()

        # 批量化处理：复制输入以便并行采样
        batch_size = x.shape[0]

        if num_samples <= CHUNCK_SIZE:  # 小样本数时批量处理
            x_expanded = x.unsqueeze(0).repeat(num_samples, 1, *[1] * (x.dim() - 1))
            x_batch = x_expanded.view(-1, *x.shape[1:])

            with torch.no_grad():
                predictions_batch = self.backbone(x_batch)
                predictions_batch = self.activation(predictions_batch)

            prediction_samples = predictions_batch.view(num_samples, batch_size, -1)
        else:  # 大样本数时分批处理
            samples = []
            chunk_size = min(CHUNCK_SIZE, num_samples)

            for i in range(0, num_samples, chunk_size):
                current_chunk = min(chunk_size, num_samples - i)
                x_chunk = x.unsqueeze(0).repeat(current_chunk, 1, *[1] * (x.dim() - 1))
                x_batch = x_chunk.view(-1, *x.shape[1:])

                with torch.no_grad():
                    chunk_pred = self.backbone(x_batch)
                    chunk_pred = self.activation(chunk_pred)

                samples.append(chunk_pred.view(current_chunk, batch_size, -1))

            prediction_samples = torch.cat(samples, dim=0)

        # 计算统计量
        predictive_mean = torch.mean(prediction_samples, dim=0)
        predictive_variance = torch.var(prediction_samples, dim=0, unbiased=True)

        return BNNOutput(
            predictive_mean=predictive_mean,
            predictive_variance=predictive_variance,
            prediction_samples=prediction_samples,
        )


class DeepEnsemble(BaseModule):
    """Deep Ensemble 不确定性估计

    通过训练多个独立初始化的神经网络并对其预测进行集成来估计不确定性。
    每个模型代表后验分布的一个近似样本。
    """

    BNN_TYPE = 'DeepEnsemble'

    def __init__(
        self,
        models: typing.Sequence[nn.Module],
        activation: Optional[
            Literal[
                'relu', 'leaky_relu', 'prelu', 'gelu', 'silu', 'tanh', 'sigmoid', 'swish'
            ]
        ]
        | None = None,
    ):
        super().__init__()
        # 使用 ModuleList 管理多个模型
        self.backbone = nn.ModuleList(models)

        self.num_models = len(models)
        self.models = self.backbone  # 别名，便于访问
        self.activation = get_activation(activation, return_type='function') or (
            lambda x: x
        )

    def forward(self, x: Tensor) -> Tensor:
        """集成预测（所有模型的平均）"""
        predictions = []

        for model in self.models:
            pred = model(x)
            pred = self.activation(pred)  # 应用激活函数
            predictions.append(pred)

        # 返回平均预测
        ensemble_pred = torch.stack(predictions, dim=0)
        return torch.mean(ensemble_pred, dim=0)

    def predict_bnn(
        self,
        x: Tensor,
        # num_samples: Optional[int] = None,  # Deep Ensemble 不需要额外采样
    ) -> BNNOutput:
        """Deep Ensemble 不确定性估计"""
        predictions = []

        # 获取每个模型的预测
        for model in self.models:
            model.eval()
            with torch.no_grad():
                pred = model(x)
                pred = self.activation(pred)
                predictions.append(pred)

        # 堆叠预测 [num_models, batch_size, ...]
        prediction_samples = torch.stack(predictions, dim=0)

        # 计算统计量
        predictive_mean = torch.mean(prediction_samples, dim=0)
        predictive_variance = torch.var(prediction_samples, dim=0)

        return BNNOutput(
            predictive_mean=predictive_mean,
            predictive_variance=predictive_variance,
            prediction_samples=prediction_samples,
        )

    def get_model(self, index: int) -> nn.Module:
        """获取指定索引的模型"""
        if index >= self.num_models:
            raise IndexError(f'模型索引 {index} 超出范围 [0, {self.num_models-1}]')
        return self.models[index]
